# $Id$
#
#  Copyright (C) 2003-2005  Rational Discovery LLC
#         All Rights Reserved
#
""" This is a rough coverage test of the python wrapper

it's intended to be shallow, but broad

"""
import os
import pickle
import unittest

from rdkit import RDConfig
from rdkit.RDLogger import logger

logger = logger()
from rdkit import Chem, DataStructs
from rdkit.Chem import FragmentCatalog


class TestCase(unittest.TestCase):

  def setUp(self):
    self.fName = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'FragCatalog', 'test_data',
                              'funcGroups.txt')
    self.smiName = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'FragCatalog', 'test_data',
                                'mols.smi')

  def test0Params(self):
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName, 1.0e-8)

    ctype = fparams.GetTypeString()
    self.assertEqual(ctype, "Fragment Catalog Parameters")
    self.assertEqual(fparams.GetLowerFragLength(), 1)
    self.assertEqual(fparams.GetUpperFragLength(), 6)

    ngps = fparams.GetNumFuncGroups()
    self.assertEqual(ngps, 15)
    for i in range(ngps):
      mol = fparams.GetFuncGroup(i)

  def test1Catalog(self):
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName, 1.0e-8)
    fcat = FragmentCatalog.FragCatalog(fparams)
    self.assertEqual(fcat.GetNumEntries(), 0)
    self.assertEqual(fcat.GetFPLength(), 0)

    nparams = fcat.GetCatalogParams()
    self.assertEqual(nparams.GetLowerFragLength(), 1)
    self.assertEqual(nparams.GetUpperFragLength(), 6)

  def test2Generator(self):
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName, 1.0e-8)
    fcat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    suppl = Chem.SmilesMolSupplier(self.smiName, " ", 0, 1, 0)
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, fcat)
    self.assertEqual(fcat.GetNumEntries(), 21)
    self.assertEqual(fcat.GetFPLength(), 21)
    for id in range(fcat.GetNumEntries()):
      self.assertEqual(fcat.GetEntryBitId(id), id)
      self.assertEqual(fcat.GetEntryOrder(id), fcat.GetBitOrder(id))
      self.assertEqual(fcat.GetEntryDescription(id), fcat.GetBitDescription(id))
      self.assertEqual(tuple(fcat.GetEntryFuncGroupIds(id)), tuple(fcat.GetBitFuncGroupIds(id)))

  def test3FPgenerator(self):
    with open(self.smiName, 'r') as smiF:
      smiLines = smiF.readlines()
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName)
    fcat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    suppl = Chem.SmilesMolSupplier(self.smiName, " ", 0, 1, 0)
    smiles = []
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, fcat)
      smiles.append(Chem.MolToSmiles(mol))
    self.assertEqual(fcat.GetNumEntries(), 21)
    self.assertEqual(fcat.GetFPLength(), 21)
    fpgen = FragmentCatalog.FragFPGenerator()
    obits = [3, 2, 3, 3, 2, 3, 5, 5, 5, 4, 5, 6]
    obls = [(0, 1, 2), (1, 3), (1, 4, 5), (1, 6, 7), (0, 8), (0, 6, 9), (0, 1, 2, 3, 10),
            (0, 1, 2, 8, 11), (1, 3, 4, 5, 12), (1, 4, 5, 13), (1, 3, 6, 7, 14),
            (0, 1, 6, 7, 9, 15)]
    for i in range(len(smiles)):
      smi = smiles[i]
      mol = Chem.MolFromSmiles(smi)
      fp = fpgen.GetFPForMol(mol, fcat)
      if i < len(obits):
        self.assertEqual(fp.GetNumOnBits(), obits[i])
      obl = fp.GetOnBits()
      if i < len(obls):
        self.assertEqual(tuple(obl), obls[i])

  def test4Serialize(self):
    with open(self.smiName, 'r') as smiF:
      smiLines = smiF.readlines()
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName)
    fcat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    suppl = Chem.SmilesMolSupplier(self.smiName, " ", 0, 1, 0)
    smiles = []
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, fcat)
      smiles.append(Chem.MolToSmiles(mol))
    self.assertEqual(fcat.GetNumEntries(), 21)
    self.assertEqual(fcat.GetFPLength(), 21)
    pkl = pickle.dumps(fcat)
    fcat2 = pickle.loads(pkl)
    self.assertEqual(fcat2.GetNumEntries(), 21)
    self.assertEqual(fcat2.GetFPLength(), 21)
    fpgen = FragmentCatalog.FragFPGenerator()
    for i in range(len(smiles)):
      smi = smiles[i]
      mol = Chem.MolFromSmiles(smi)
      fp1 = fpgen.GetFPForMol(mol, fcat)
      fp2 = fpgen.GetFPForMol(mol, fcat2)
      self.assertEqual(fp1.GetNumOnBits(), fp2.GetNumOnBits())
      obl1 = fp1.GetOnBits()
      obl2 = fp2.GetOnBits()
      self.assertEqual(tuple(obl1), tuple(obl2))

  def test5FPsize(self):
    with open(self.smiName, 'r') as smiF:
      smiLines = smiF.readlines()
    fparams = FragmentCatalog.FragCatParams(6, 6, self.fName)
    fcat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    suppl = [Chem.MolFromSmiles('C1CCCOC1O')]
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, fcat)
    self.assertEqual(fcat.GetFPLength(), 1)
    for i in range(fcat.GetFPLength()):
      self.assertEqual(fcat.GetBitOrder(i), 6)
      self.assertEqual(fcat.GetBitDescription(i), "C1CC<-O>OCC1")
      self.assertEqual(tuple(fcat.GetBitFuncGroupIds(i)), (1, ))

  def test6DownEntries(self):
    fparams = FragmentCatalog.FragCatParams(1, 6, self.fName, 1.0e-8)
    fcat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    suppl = Chem.SmilesMolSupplier(self.smiName, " ", 0, 1, 0)
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, fcat)
    self.assertEqual(fcat.GetNumEntries(), 21)
    self.assertEqual(fcat.GetFPLength(), 21)
    self.assertEqual(tuple(fcat.GetEntryDownIds(0)), (2, 8, 9, 16))
    self.assertEqual(tuple(fcat.GetEntryDownIds(1)), (2, 3, 5, 7))

  def test7Issue116(self):
    smiList = ['Cc1ccccc1']
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(smiList), ',', 0, -1, 0)
    fparams = FragmentCatalog.FragCatParams(2, 2, self.fName, 1.0e-8)
    cat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, cat)
    self.assertEqual(cat.GetFPLength(), 2)
    self.assertEqual(cat.GetBitDescription(0), 'ccC')
    fpgen = FragmentCatalog.FragFPGenerator()
    mol = Chem.MolFromSmiles('Cc1ccccc1')
    fp = fpgen.GetFPForMol(mol, cat)
    self.assertEqual(fp[0], 1)
    self.assertEqual(fp[1], 1)

    mol = Chem.MolFromSmiles('c1ccccc1-c1ccccc1')
    fp = fpgen.GetFPForMol(mol, cat)
    self.assertEqual(fp[0], 0)
    self.assertEqual(fp[1], 1)

  def test8Issue118(self):
    smiList = ['CCN(C(N)=O)N=O']
    fName = os.path.join(RDConfig.RDDataDir, 'FunctionalGroups.txt')
    suppl = Chem.SmilesMolSupplierFromText('\n'.join(smiList), ',', 0, -1, 0)
    fparams = FragmentCatalog.FragCatParams(2, 4, fName, 1.0e-8)
    cat = FragmentCatalog.FragCatalog(fparams)
    fgen = FragmentCatalog.FragCatGenerator()
    for mol in suppl:
      nent = fgen.AddFragsFromMol(mol, cat)
    self.assertEqual(cat.GetFPLength(), 1)
    self.assertEqual(cat.GetBitDescription(0), 'CCN(<-C(=O)N>)<-N=O>')


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2003-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for database connection objects

"""

import doctest
import os
import shutil
import tempfile
import traceback
import unittest

from rdkit import RDConfig
from rdkit.Dbase import StorageUtils


class TestCase(unittest.TestCase):

  def testDoctest(self):
    if RDConfig.useSqlLite:
      fd, tempName = tempfile.mkstemp(suffix='sqlt')
      StorageUtils.fd = fd
      StorageUtils.tempDbName = tempName
      shutil.copyfile(RDConfig.RDTestDatabase, StorageUtils.tempDbName)
    else:
      StorageUtils.tempDbName = '::RDTests'
    failed, _ = doctest.testmod(StorageUtils)
    if RDConfig.useSqlLite and os.path.exists(StorageUtils.tempDbName):
      os.close(StorageUtils.fd)
      try:
        os.unlink(StorageUtils.tempDbName)
      except Exception:
        traceback.print_exc()
    self.assertFalse(failed)

  def test_ValidateRDId(self):
    self.assertEqual(StorageUtils.ValidateRDId('RDCmpd-bogus-000-9'), False)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2011-2021  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import os
import pathlib
import re
import sys
import tempfile
import unittest

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Draw
from rdkit.Chem import rdDepictor
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem.Draw import rdMolDraw2D

try:
  from rdkit.Chem.Draw import IPythonConsole
except ImportError:
  IPythonConsole = None


class TestCase(unittest.TestCase):
  showAllImages = False

  def setUp(self):
    if IPythonConsole is not None and Draw.MolsToGridImage == IPythonConsole.ShowMols:
      IPythonConsole.UninstallIPythonRenderer()
    self.mol = Chem.MolFromSmiles('c1c(C[15NH3+])ccnc1[C@](Cl)(Br)[C@](Cl)(Br)F')

    # For testMolsMatrixToLinear and testMolsMatrixToLinear (which test MolsMatrixToGridImage and its helper _MolsNestedToLinear)
    s = "NC(C)C(=O)"
    mol = Chem.MolFromSmiles(s)
    natoms = mol.GetNumAtoms()
    nbonds = mol.GetNumBonds()

    # Set up matrix with oligomer count for the molecules
    # Should produce this grid:
    # NC(C)C(=O)
    #                                NC(C)C(=O)NC(C)C(=O)
    # NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)                      NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)NC(C)C(=O)
    repeats = [[1], [0, 2], [3, 0, 4]]

    # Create molecule if there are 1 or more oligomers;
    # otherwise, use None for molecule because drawing is distorted if use Chem.MolFromSmiles("")
    self.molsMatrix = [[Chem.MolFromSmiles(s * count) if count else None for count in row]
                       for row in repeats]

    self.legendsMatrix = [[str(count) + " unit(s)" for count in row] for row in repeats]

    def ithItemList(nunits, itemsPerUnit, i=0):
      return [((n * itemsPerUnit) + i) for n in range(nunits)]

    self.highlightAtomListsMatrix = [[ithItemList(count, natoms, 0) for count in row]
                                     for row in repeats]

    # Another bond is created when molecule is oligomerized, so to keep the bond type consistent,
    #   make items per unit one more than the number of bonds
    self.highlightBondListsMatrix = [[ithItemList(count, nbonds + 1, 1) for count in row]
                                     for row in repeats]

    # Parametrize tests: In addition to supplying molsMatrix, supply 0-3 other matrices
    # col labels: legendsMatrix, highlightAtomListsMatrix, highlightBondListsMatrix
    # Zero other matrices: 1 parameter set
    self.paramSets = [
      (None, None, None),
      # One other matrix: 3 parameter sets
      (self.legendsMatrix, None, None),
      (None, self.highlightAtomListsMatrix, None),
      (None, None, self.highlightBondListsMatrix),
      # Two other matrices: 3 parameter sets
      (self.legendsMatrix, self.highlightAtomListsMatrix, None),
      (self.legendsMatrix, None, self.highlightBondListsMatrix),
      (None, self.highlightAtomListsMatrix, self.highlightBondListsMatrix),
      # All three other matrices: 1 parameter set
      (self.legendsMatrix, self.highlightAtomListsMatrix, self.highlightBondListsMatrix),
    ]

  def test_interactive(self):
    # We avoid checking in the code with development flag set
    self.assertFalse(self.showAllImages)

  def _testMolToFile(self):
    try:
      fhdl, fn = tempfile.mkstemp(suffix='.png')
      # mkstemp returns a file handle that we don't need; close it
      os.close(fhdl)
      fhdl = None
      self.assertEqual(os.path.getsize(fn), 0)
      Draw.MolToFile(self.mol, fn)
      self.assertNotEqual(os.path.getsize(fn), 0)
    finally:
      os.remove(fn)

  @unittest.skipIf(not hasattr(rdMolDraw2D, 'MolDraw2DCairo'), 'Skipping cairo test')
  def testCairoFile(self):
    self._testMolToFile()

  def _testMolToImage(self, mol=None, kekulize=True, options=None, showImage=False, **kwargs):
    mol = mol or self.mol
    img = Draw.MolToImage(mol, size=(300, 300), kekulize=kekulize, options=options, **kwargs)
    self.assertTrue(img)
    self.assertEqual(img.size[0], 300)
    self.assertEqual(img.size[1], 300)
    if self.showAllImages or showImage:
      img.show()

  @unittest.skipIf(not hasattr(rdMolDraw2D, 'MolDraw2DCairo'), 'Skipping cairo test')
  def testCairoImage(self):
    self._testMolToImage()

  @unittest.skipIf(not hasattr(rdMolDraw2D, 'MolDraw2DCairo'), 'Skipping cairo test')
  def testCairoImageDash(self):
    os.environ['RDKIT_CANVAS'] = 'cairo'
    self._testMolToImage(kekulize=False)

  @unittest.skipIf(not hasattr(rdMolDraw2D, 'MolDraw2DCairo'), 'Skipping test requiring cairo')
  def testSpecialCases(self):
    options = rdMolDraw2D.MolDrawOptions()
    options.atomLabelDeuteriumTritium = True
    self._testMolToImage(mol=Chem.MolFromSmiles('[2H][C@]([3H])(C)F'), options=options)
    # shared rings
    self._testMolToImage(mol=Chem.MolFromSmiles('c1cccc2cc(cccc3)c3cc21'))
    self._testMolToImage(mol=Chem.MolFromSmiles('C1=CC=CC=CC=C1'))
    self._testMolToImage(mol=Chem.MolFromSmiles('C=C=C'))
    self._testMolToImage(mol=Chem.MolFromSmiles('CC#N'), showImage=False)
    self._testMolToImage(mol=Chem.MolFromSmiles('[CH2-][C-2]C[CH3+][CH5+2]'))
    self._testMolToImage(mol=Chem.MolFromSmiles('[Na+].[OH-]'))
    self._testMolToImage(mol=Chem.MolFromSmiles('c1ccccc1c1ccccc1'),
                         highlightAtoms=(0, 1, 2, 3, 4, 5, 6))
    self._testMolToImage(mol=Chem.MolFromSmiles('c1ccccc1c1ccccc1'),
                         highlightBonds=(0, 2, 4, 6, 8, 10))
    self._testMolToImage(mol=Chem.MolFromSmiles('c1ccccc1c1ccc(cc1)c1ccc(cc1)c1ccc(cc1)'))

  @unittest.skipIf(not hasattr(rdMolDraw2D, 'MolDraw2DCairo'), 'Skipping test requiring cairo')
  def testGithubIssue86(self):
    # Assert that drawing code doesn't modify wedge bonds
    mol = Chem.MolFromSmiles('F[C@H](Cl)Br')
    for b in mol.GetBonds():
      self.assertEqual(b.GetBondDir(), Chem.BondDir.NONE)

    rdDepictor.Compute2DCoords(mol)
    img = Draw.MolToImage(mol, kekulize=False)
    self.assertTrue(img)
    # img.show()
    for b in mol.GetBonds():
      self.assertEqual(b.GetBondDir(), Chem.BondDir.NONE)

    Chem.WedgeMolBonds(mol, mol.GetConformer())
    obds = [x.GetBondDir() for x in mol.GetBonds()]
    self.assertEqual(obds.count(Chem.BondDir.NONE), 2)
    img = Draw.MolToImage(mol, kekulize=False)
    self.assertTrue(img)
    # img.show()
    nbds = [x.GetBondDir() for x in mol.GetBonds()]
    self.assertEqual(obds, nbds)

  def testGridSVG(self):
    mols = [Chem.MolFromSmiles('NC(C)C(=O)' * x) for x in range(10)]
    legends = ['mol-%d' % x for x in range(len(mols))]
    svg = Draw.MolsToGridImage(mols, legends=legends, molsPerRow=3, subImgSize=(200, 200),
                               useSVG=True)
    self.assertIn("width='600px' height='800px'", svg)
    svg = Draw.MolsToGridImage(mols, legends=legends, molsPerRow=4, subImgSize=(200, 200),
                               useSVG=True)
    self.assertIn("width='800px' height='600px'", svg)
    svg = Draw.MolsToGridImage(mols, legends=legends, molsPerRow=3, subImgSize=(300, 300),
                               useSVG=True)
    self.assertIn("width='900px' height='1200px'", svg)
    self.assertNotIn("class='note'", svg)
    dopts = Draw.rdMolDraw2D.MolDrawOptions()
    dopts.addAtomIndices = True
    svg = Draw.MolsToGridImage(mols, legends=legends, molsPerRow=3, subImgSize=(300, 300),
                               useSVG=True, drawOptions=dopts)
    self.assertIn("class='note'", svg)

  def testMolsMatrixToLinear(self):
    mols, molsPerRow, legends, highlightAtomLists, highlightBondLists = Draw._MolsNestedToLinear(
      self.molsMatrix, self.legendsMatrix, self.highlightAtomListsMatrix,
      self.highlightBondListsMatrix)

    nrows = len(self.molsMatrix)

    def _nestedOrder(self, molsMatrix, legendsMatrix, highlightAtomListsMatrix,
                     highlightBondListsMatrix):
      for r, row in enumerate(molsMatrix):
        for c, item in enumerate(row):
          linearIndex = (r * molsPerRow) + c
          # Test that items in 2D list are in correct position in 1D list
          self.assertTrue(mols[linearIndex] == item)
          # Test that 1D list items are not lists
          self.assertFalse(isinstance(item, list))

      # Other three matrices (legends; atom and bond highlighting) need not be supplied;
      #   only test each if it's supplied
      if legendsMatrix is not None:
        for r, row in enumerate(legendsMatrix):
          for c, item in enumerate(row):
            linearIndex = (r * molsPerRow) + c
            # Test that items in 2D list are in correct position in 1D list
            self.assertTrue(legends[linearIndex] == item)
            # Test that 1D list items are not lists
            self.assertFalse(isinstance(item, list))

      if highlightAtomListsMatrix is not None:
        for r, row in enumerate(highlightAtomListsMatrix):
          for c, item in enumerate(row):
            linearIndex = (r * molsPerRow) + c
            # Test that items in 2D list are in correct position in 1D list
            self.assertTrue(highlightAtomLists[linearIndex] == item)
            # For highlight parameters, entries are lists, so check that sub-items are not lists
            for subitem in item:
              self.assertFalse(isinstance(subitem, list))

      if highlightBondListsMatrix is not None:
        for r, row in enumerate(highlightBondListsMatrix):
          for c, item in enumerate(row):
            linearIndex = (r * molsPerRow) + c
            # Test that items in 2D list are in correct position in 1D list
            self.assertTrue(highlightBondLists[linearIndex] == item)
            # For highlight parameters, entries are lists, so check that sub-items are not lists
            for subitem in item:
              self.assertFalse(isinstance(subitem, list))

      # Test that 1D list has the correct length
      self.assertTrue(len(mols) == nrows * molsPerRow)

    # Parametrize tests: In addition to supplying molsMatrix, supply 0-3 other matrices
    for paramSet in self.paramSets:
      _nestedOrder(self, self.molsMatrix, *paramSet)

    ## Test that exceptions are thrown appropriately

    # Test that supplying a non-nested list raises a ValueError
    # Set up non-nested lists = first sublist of nested lists
    molsNotNested = self.molsMatrix[0]
    legendsNotNested = self.legendsMatrix[0]
    highlightAtomListsNotNested = self.highlightAtomListsMatrix[0]
    highlightBondListsNotNested = self.highlightBondListsMatrix[0]

    with self.assertRaises(TypeError):
      Draw._MolsNestedToLinear(molsNotNested)

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix, legendsMatrix=legendsNotNested)

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightAtomListsMatrix=highlightAtomListsNotNested)

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightBondListsMatrix=highlightBondListsNotNested)

    # Test that raises ValueError if other matrices aren't same size (# rows) as molsMatrix
    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix, legendsMatrix=self.legendsMatrix[0:1])

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightAtomListsMatrix=self.highlightAtomListsMatrix[0:1])

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightBondListsMatrix=self.highlightBondListsMatrix[0:1])

    # Test that raises ValueError if other matrices' rows aren't same length as molsMatrix's corresponding row
    # Remove last element from first row of each other matrix
    legendsMatrixShortRow0 = [self.legendsMatrix[0][0:-1]] + [self.legendsMatrix[1:]]
    highlightAtomListsMatrixShortRow0 = [self.highlightAtomListsMatrix[0][0:-1]
                                         ] + [self.highlightAtomListsMatrix[1:]]
    highlightBondListsMatrixShortRow0 = [self.highlightBondListsMatrix[0][0:-1]
                                         ] + [self.highlightBondListsMatrix[1:]]

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix, legendsMatrix=legendsMatrixShortRow0)

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightAtomListsMatrix=highlightAtomListsMatrixShortRow0)

    with self.assertRaises(ValueError):
      Draw._MolsNestedToLinear(molsMatrix=self.molsMatrix,
                               highlightBondListsMatrix=highlightBondListsMatrixShortRow0)

  def testMolsMatrixToGridImage(self):
    subImgSize = (200, 200)

    kwargsValue = Draw.rdMolDraw2D.MolDrawOptions()
    kwargsValue.addAtomIndices = True

    # Tests are that running Draw.MolsMatrixToGridImage doesn't give an error
    # with any combination of parameters supplied (or not)
    # by parametrizing tests: In addition to supplying molsMatrix, supply 0-3 other matrices
    for (legendsMatrix, highlightAtomListsMatrix, highlightBondListsMatrix) in self.paramSets:
      for useSVG in (True, False):
        if not (useSVG or hasattr(rdMolDraw2D, 'MolDraw2DCairo')):
          continue
        for returnPNG in (True, False):
          dwgSubImgSizeNokwargs = Draw.MolsMatrixToGridImage(
            self.molsMatrix, subImgSize, legendsMatrix=legendsMatrix,
            highlightAtomListsMatrix=highlightAtomListsMatrix,
            highlightBondListsMatrix=highlightBondListsMatrix, useSVG=useSVG, returnPNG=returnPNG)
          if useSVG:
            self.assertIn('<svg', dwgSubImgSizeNokwargs)
          dwgSubImgSizeKwargs = Draw.MolsMatrixToGridImage(
            self.molsMatrix, subImgSize, legendsMatrix=legendsMatrix,
            highlightAtomListsMatrix=highlightAtomListsMatrix,
            highlightBondListsMatrix=highlightBondListsMatrix, useSVG=useSVG, returnPNG=returnPNG,
            drawOptions=kwargsValue)
          if useSVG:
            self.assertIn('<svg', dwgSubImgSizeKwargs)
          dwgNosubImgSizeNokwargs = Draw.MolsMatrixToGridImage(
            self.molsMatrix, legendsMatrix=legendsMatrix,
            highlightAtomListsMatrix=highlightAtomListsMatrix,
            highlightBondListsMatrix=highlightBondListsMatrix, useSVG=useSVG, returnPNG=returnPNG)
          if useSVG:
            self.assertIn('<svg', dwgNosubImgSizeNokwargs)
          dwgNosubImgSizeKwargs = Draw.MolsMatrixToGridImage(
            self.molsMatrix, legendsMatrix=legendsMatrix,
            highlightAtomListsMatrix=highlightAtomListsMatrix,
            highlightBondListsMatrix=highlightBondListsMatrix, useSVG=useSVG, returnPNG=returnPNG,
            drawOptions=kwargsValue)
          if useSVG:
            self.assertIn('<svg', dwgNosubImgSizeKwargs)

  def testDrawMorgan(self):
    m = Chem.MolFromSmiles('c1ccccc1CC1CC1')
    bi = {}
    _ = rdMolDescriptors.GetMorganFingerprintAsBitVect(m, radius=2, bitInfo=bi)
    self.assertTrue(872 in bi)

    svg1 = Draw.DrawMorganBit(m, 872, bi)
    aid, r = bi[872][0]
    svg2 = Draw.DrawMorganEnv(m, aid, r)
    self.assertEqual(svg1, svg2)
    self.assertTrue("style='fill:#CCCCCC;" in svg1)
    self.assertTrue("style='fill:#E5E533;" in svg1)
    self.assertTrue("style='fill:#9999E5;" in svg1)

    svg1 = Draw.DrawMorganBit(m, 872, bi, centerColor=None)
    aid, r = bi[872][0]
    svg2 = Draw.DrawMorganEnv(m, aid, r, centerColor=None)
    self.assertEqual(svg1, svg2)
    self.assertTrue("style='fill:#CCCCCC;" in svg1)
    self.assertTrue("style='fill:#E5E533;" in svg1)
    self.assertFalse("style='fill:#9999E5;" in svg1)
    with self.assertRaises(KeyError):
      Draw.DrawMorganBit(m, 32, bi)

    if hasattr(Draw, 'MolDraw2DCairo') and hasattr(Draw, 'MolFromPNGString'):
      # Github #3796: make sure we aren't trying to generate metadata:
      png = Draw.DrawMorganBit(m, 872, bi, useSVG=False)
      self.assertIn(b'PNG', png)
      self.assertIsNone(Chem.MolFromPNGString(png))

  def testDrawRDKit(self):
    m = Chem.MolFromSmiles('c1ccccc1CC1CC1')
    bi = {}
    _ = Chem.RDKFingerprint(m, maxPath=5, bitInfo=bi)
    self.assertTrue(1553 in bi)
    svg1 = Draw.DrawRDKitBit(m, 1553, bi)
    path = bi[1553][0]
    svg2 = Draw.DrawRDKitEnv(m, path)
    self.assertEqual(svg1, svg2)
    self.assertTrue("style='fill:#E5E533;" in svg1)
    self.assertFalse("style='fill:#CCCCCC;" in svg1)
    self.assertFalse("style='fill:#9999E5;" in svg1)
    with self.assertRaises(KeyError):
      Draw.DrawRDKitBit(m, 32, bi)

    if hasattr(Draw, 'MolDraw2DCairo') and hasattr(Draw, 'MolFromPNGString'):
      # Github #3796: make sure we aren't trying to generate metadata:
      png = Draw.DrawRDKitBit(m, 1553, bi, useSVG=False)
      self.assertIn(b'PNG', png)
      self.assertIsNone(Chem.MolFromPNGString(png))

  def testDrawReaction(self):
    # this shouldn't throw an exception...
    rxn = AllChem.ReactionFromSmarts(
      "[c;H1:3]1:[c:4]:[c:5]:[c;H1:6]:[c:7]2:[nH:8]:[c:9]:[c;H1:1]:[c:2]:1:2.O=[C:10]1[#6;H2:11][#6;H2:12][N:13][#6;H2:14][#6;H2:15]1>>[#6;H2:12]3[#6;H1:11]=[C:10]([c:1]1:[c:9]:[n:8]:[c:7]2:[c:6]:[c:5]:[c:4]:[c:3]:[c:2]:1:2)[#6;H2:15][#6;H2:14][N:13]3"
    )
    _ = Draw.ReactionToImage(rxn, useSVG=True)

  def testGithub3762(self):
    m = Chem.MolFromSmiles('CC(=O)O')
    ats = [1, 2, 3]
    svg1 = Draw._moltoSVG(m, (250, 200), ats, "", False)
    svg2 = Draw._moltoSVG(m, (250, 200), ats, "", False, highlightBonds=[])
    with open('testGithub_3762_1.svg', 'w') as f:
      f.write(svg1)
    with open('testGithub_3762_2.svg', 'w') as f:
      f.write(svg2)

    re_str = r"path class='bond-\d atom-\d atom-\d' d='M \d+.\d+,\d+.\d+ L \d+.\d+,\d+.\d+ L \d+.\d+,\d+.\d+ L \d+.\d+,\d+.\d+ Z' style='fill:#FF7F7F;"
    patt = re.compile(re_str)
    self.assertEqual(len(patt.findall(svg1)), 2)
    self.assertEqual(len(patt.findall(svg2)), 0)

    pathlib.Path('testGithub_3762_1.svg').unlink()
    pathlib.Path('testGithub_3762_2.svg').unlink()

  def testGithub5863(self):
    smiles = "C[C@]12C[C@H](O)[C@H]3[C@@H](CCC4=CC(=O)C=C[C@@]43C)[C@@H]1CC[C@]2(O)C(=O)CO"
    mol = Chem.MolFromSmiles(smiles)

    info = {}
    rdMolDescriptors.GetMorganFingerprint(mol, radius=2, bitInfo=info, useChirality=True)
    bitId = 1236726849
    # this should run without generating an exception:
    Draw.DrawMorganBit(mol, bitId, info)


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2002-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""
unit testing code for calculations in rdkit.Chem.MolSurf
"""

import os.path
import unittest
from collections import namedtuple

from rdkit import Chem, RDConfig
from rdkit.Chem import MolSurf

doLong = False
_TestData = namedtuple('_TestData', 'lineNo,smiles,mol,expected')


class TestCase(unittest.TestCase):
  dataNCI200 = os.path.join(RDConfig.RDDataDir, 'NCI', 'first_200.tpsa.csv')
  dataNCI5000 = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'NCI_5K_TPSA.csv')
  dataTPSAregr = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'tpsa_regr.csv')

  @classmethod
  def readNCI_200(cls):
    for data in readPSAtestData(cls.dataNCI200):
      yield data

  @classmethod
  def readNCI_5000(cls):
    for data in readPSAtestData(cls.dataNCI5000):
      yield data

  @classmethod
  def readTPSAregres(cls):
    for data in readPSAtestData(cls.dataTPSAregr):
      yield data

  def testTPSAShort(self):
    for data in self.readNCI_200():
      calc = MolSurf.TPSA(data.mol)
      self.assertAlmostEqual(
        calc, data.expected, delta=1e-4,
        msg='bad TPSA for SMILES {0.smiles} ({1:.2f} != {0.expected:.2f})'.format(data, calc))

  def testTPSALong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readNCI_5000():
      try:
        calc = MolSurf.TPSA(data.mol)
      except Exception:
        raise AssertionError(
          'Line {0.lineNo}: TPSA Calculation failed for SMILES {0.smiles}'.format(data))
      self.assertAlmostEqual(
        calc, data.expected, delta=1e-4,
        msg='bad TPSA for SMILES {0.smiles} ({1:.2f} != {0.expected:.2f})'.format(data, calc))

  def testTPSALongNCI(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readTPSAregres():
      try:
        calc = MolSurf.TPSA(data.mol)
      except Exception:
        raise AssertionError(
          'Line {0.lineNo}: TPSA Calculation failed for SMILES {0.smiles}'.format(data))
      self.assertAlmostEqual(
        calc, data.expected, delta=1e-4,
        msg='bad TPSA for SMILES {0.smiles} ({1:.2f} != {0.expected:.2f})'.format(data, calc))

  def testHsAndTPSA(self):
    """
     testing the impact of Hs in the graph on PSA calculations
     This was sf.net issue 1969745
    """
    mol = Chem.MolFromSmiles('c1c[nH]cc1')
    molH = Chem.AddHs(mol)
    psa = MolSurf.TPSA(mol)
    psaH = MolSurf.TPSA(molH)

    if psa != psaH:
      psac = MolSurf.rdMolDescriptors._CalcTPSAContribs(mol)
      psaHc = MolSurf.rdMolDescriptors._CalcTPSAContribs(molH)
      for i, v in enumerate(psac):
        print('\t', i, '\t', v, '\t', psaHc[i])
      while i < len(psaHc):
        print('\t\t\t', psaHc[i])
        i += 1
    self.assertEqual(psa, psaH)

    for data in self.readNCI_200():
      mol = Chem.AddHs(data.mol)
      calc = MolSurf.TPSA(mol)
      self.assertAlmostEqual(
        calc, data.expected, delta=1e-4,
        msg='bad TPSA for SMILES {0.smiles} ({1:.2f} != {0.expected:.2f})'.format(data, calc))

    if doLong:
      for data in self.readNCI_5000():
        mol = Chem.AddHs(data.mol)
        calc = MolSurf.TPSA(mol)
        self.assertAlmostEqual(
          calc, data.expected, delta=1e-4,
          msg='bad TPSA for SMILES {0.smiles} ({1:.2f} != {0.expected:.2f})'.format(data, calc))


class TestCase_descriptorRegression(unittest.TestCase):

  def __testDesc(self, fileN, col, func):
    for data in readRegressionData(fileN, col):
      if abs(data.expected - 666.0) < 1e-4:
        print(data)
        raise AssertionError('check this data entry')
      try:
        val = func(data.mol)
      except Exception:
        val = 666
      self.assertAlmostEqual(
        val, data.expected, delta=1e-4,
        msg='line {0.lineNo}, mol {0.smiles} (calc = {1}) should have val = {0.expected}'.format(
          data, val))

  def testLabuteASALong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    col = 6
    self.__testDesc('PP_descrs_regress.csv', col, lambda x: MolSurf.LabuteASA(x, includeHs=1))
    self.__testDesc('PP_descrs_regress.2.csv', col, lambda x: MolSurf.LabuteASA(x, includeHs=1))

  def testTPSALong(self):
    col = 28
    self.__testDesc('PP_descrs_regress.csv', col, MolSurf.TPSA)
    if doLong:
      self.__testDesc('PP_descrs_regress.2.csv', col, MolSurf.TPSA)

  def testMOELong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    fName = 'PP_descrs_regress.VSA.csv'
    self.__testDesc(fName, 1, MolSurf.SMR_VSA1)
    self.__testDesc(fName, 2, MolSurf.SMR_VSA10)
    self.__testDesc(fName, 3, MolSurf.SMR_VSA2)
    self.__testDesc(fName, 4, MolSurf.SMR_VSA3)
    self.__testDesc(fName, 5, MolSurf.SMR_VSA4)
    self.__testDesc(fName, 6, MolSurf.SMR_VSA5)
    self.__testDesc(fName, 7, MolSurf.SMR_VSA6)
    self.__testDesc(fName, 8, MolSurf.SMR_VSA7)
    self.__testDesc(fName, 9, MolSurf.SMR_VSA8)
    self.__testDesc(fName, 10, MolSurf.SMR_VSA9)
    self.__testDesc(fName, 11, MolSurf.SlogP_VSA1)
    self.__testDesc(fName, 12, MolSurf.SlogP_VSA10)
    self.__testDesc(fName, 13, MolSurf.SlogP_VSA11)
    self.__testDesc(fName, 14, MolSurf.SlogP_VSA12)

    fName = 'PP_descrs_regress.VSA.2.csv'
    self.__testDesc(fName, 1, MolSurf.SMR_VSA1)
    self.__testDesc(fName, 2, MolSurf.SMR_VSA10)
    self.__testDesc(fName, 11, MolSurf.SlogP_VSA1)
    self.__testDesc(fName, 12, MolSurf.SlogP_VSA10)
    self.__testDesc(fName, 13, MolSurf.SlogP_VSA11)
    self.__testDesc(fName, 14, MolSurf.SlogP_VSA12)


class TestCase_python(unittest.TestCase):
  """ Test the Python implementation of the various descriptors """

  def test_pyTPSA(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      self.assertAlmostEqual(MolSurf.TPSA(data.mol), MolSurf._pyTPSA(molPy))

  def test_pyLabuteASA(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      self.assertAlmostEqual(MolSurf.LabuteASA(data.mol), MolSurf.pyLabuteASA(molPy))

  def test_pyPEOE_VSA_(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      for calcC, calcPy in zip(MolSurf.PEOE_VSA_(data.mol), MolSurf.pyPEOE_VSA_(molPy,
                                                                                force=False)):
        self.assertAlmostEqual(calcC, calcPy)

  def test_pySlogP_VSA_(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      for calcC, calcPy in zip(MolSurf.SlogP_VSA_(data.mol),
                               MolSurf.pySlogP_VSA_(molPy, force=False)):
        self.assertAlmostEqual(calcC, calcPy)

  def test_pySMR_VSA_(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      for calcC, calcPy in zip(MolSurf.SMR_VSA_(data.mol), MolSurf.pySMR_VSA_(molPy, force=False)):
        self.assertAlmostEqual(calcC, calcPy)

  def test_pyLabuteHelper(self):
    for data in TestCase.readNCI_200():
      molPy = Chem.MolFromSmiles(data.smiles)
      for calcC, calcPy in zip(MolSurf._LabuteHelper(data.mol), MolSurf._pyLabuteHelper(molPy)):
        self.assertAlmostEqual(calcC, calcPy)


def readPSAtestData(filename):
  """ Read test data for PSA method from file """
  with open(filename, 'r') as f:
    for lineNo, line in enumerate(f, 1):
      if line[0] == '#':
        continue
      smiles, expected = line.strip().split(',')
      mol = Chem.MolFromSmiles(smiles)
      if not mol:
        raise AssertionError('molecule construction failed on line %d' % lineNo)
      yield _TestData(lineNo, smiles, mol, float(expected))


def readRegressionData(filename, col):
  """ Return entries form regression dataset.
  Returns the line number, smiles, molecule, and the value found in column col
  """
  with open(os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', filename), 'r') as inF:
    for lineNum, line in enumerate(inF, 1):
      if line[0] == '#':
        continue
      splitL = line.split(',')
      smi = splitL[0]
      mol = Chem.MolFromSmiles(smi)
      if mol is None:
        raise AssertionError('line %d, smiles: %s' % (lineNum, smi))
      expected = float(splitL[col])
      yield _TestData(lineNum, smi, mol, expected)


if __name__ == '__main__':  # pragma: nocover
  import argparse
  import sys
  parser = argparse.ArgumentParser()
  parser.add_argument('-l', default=False, action='store_true', dest='doLong')
  args = parser.parse_args()
  doLong = args.doLong

  # Remove the -l flag if present so that it isn't interpreted by unittest.main()
  if 'l' in sys.argv:
    sys.argv.remove('-l')
  unittest.main()



import gzip
import os
import random
import sys
import time

import rdkit
from rdkit import Chem
from rdkit.Chem import AllChem, Recap
from rdkit.RDLogger import logger

logger = logger()

tests = [1] * 1001
if len(sys.argv) > 1:
  tests = [0] * 1001
  tests[1] = 1
  for x in sys.argv[1:]:
    x = int(x)
    tests[x] = 1
ts = []

sdData = gzip.open('../Data/mols.1000.sdf.gz').read()
logger.info('mols from sdf')
suppl = Chem.SDMolSupplier()
suppl.SetData(sdData)
mols = []
nMols = 0
nBad = 0
t1 = time.time()
for m in suppl:
  if m:
    nMols += 1
    mols.append(m)
  else:
    nBad += 1
t2 = time.time()
logger.info('Results1: %.2f seconds, %d passed, %d failed' % (t2 - t1, nMols, nBad))
ts.append(t2 - t1)

if tests[2]:
  lines = gzip.open('../Data/mols.1000.txt.gz').readlines()
  logger.info('mols from smiles')
  nMols = 0
  nBad = 0
  t1 = time.time()
  for line in lines:
    line = line.decode().strip().split(' ')
    m = Chem.MolFromSmiles(line[1])
    if m:
      nMols += 1
    else:
      nBad += 1
  t2 = time.time()
  logger.info('Results2: %.2f seconds, %d passed, %d failed' % (t2 - t1, nMols, nBad))
  ts.append(t2 - t1)

if tests[3] or tests[4] or tests[5]:
  pattData = gzip.open('../Data/queries.txt.gz').readlines()
  pattData = [x.decode().strip().replace('[H]', '').replace('()', '') for x in pattData]
  logger.info('patterns from smiles')
  patts = []
  nMols = 0
  t1 = time.time()
  for line in pattData:
    m = Chem.MolFromSmarts(line)
    if m:
      nMols += 1
      patts.append(m)
    else:
      nBad += 1
  t2 = time.time()
  logger.info('Results3: %.2f seconds, %d passed, %d failed' % (t2 - t1, nMols, nBad))
  ts.append(t2 - t1)
  random.seed(23)
  random.shuffle(patts)
  patts = patts[:100]

if tests[4]:
  logger.info('Matching1: HasSubstructMatch')
  t1 = time.time()
  for mol in mols:
    for patt in patts:
      mol.HasSubstructMatch(patt)
  t2 = time.time()
  logger.info('Results4: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[5]:
  logger.info('Matching2: GetSubstructMatches')
  t1 = time.time()
  for mol in mols:
    for patt in patts:
      mol.GetSubstructMatches(patt)
  t2 = time.time()
  logger.info('Results5: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[6] or tests[7] or tests[8]:
  logger.info('reading SMARTS')
  patts = []
  t1 = time.time()
  for line in open('../Data/RLewis_smarts.txt'):
    line = line.strip()
    if line == '' or line[0] == '#':
      continue
    splitL = line.split(' ')
    sma = splitL[0]
    m = Chem.MolFromSmarts(sma)
    if m:
      patts.append(m)
  t2 = time.time()
  logger.info('Results6: %.2f seconds for %d patterns' % (t2 - t1, len(patts)))
  ts.append(t2 - t1)

if tests[7]:
  logger.info('Matching3: HasSubstructMatch')
  t1 = time.time()
  for mol in mols:
    for patt in patts:
      mol.HasSubstructMatch(patt)
  t2 = time.time()
  logger.info('Results7: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[8]:
  logger.info('Matching4: GetSubstructMatches')
  t1 = time.time()
  for mol in mols:
    for patt in patts:
      mol.GetSubstructMatches(patt)
  t2 = time.time()
  logger.info('Results8: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[9]:
  logger.info('Writing: Canonical SMILES')
  t1 = time.time()
  for mol in mols:
    smi = Chem.MolToSmiles(mol)
  t2 = time.time()
  logger.info('Results9: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[10]:
  logger.info('Generate 2D coords')
  t1 = time.time()
  for mol in mols:
    AllChem.Compute2DCoords(mol)
  t2 = time.time()
  logger.info('Results10: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[11]:
  logger.info('Writing: Mol blocks')
  t1 = time.time()
  for mol in mols:
    mb = Chem.MolToMolBlock(mol)
  t2 = time.time()
  logger.info('Results11: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[12]:
  logger.info('RECAP decomposition')
  t1 = time.time()
  for mol in mols:
    d = Recap.RecapDecompose(mol)
  t2 = time.time()
  logger.info('Results12: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[13]:
  logger.info('Generate 3D coords for 50 molecules with ETKDG')
  mols3d = mols[200:250]
  t1 = time.time()
  nBad = 0
  for mol in mols3d:
    cid = AllChem.EmbedMolecule(mol, randomSeed=0xF00D, useExpTorsionAnglePrefs=True,
                                useBasicKnowledge=True)
    if cid < 0:
      nBad += 1
  t2 = time.time()
  logger.info('Results13: %.2f seconds %d failures' % (t2 - t1, nBad))
  ts.append(t2 - t1)

if tests[14]:
  logger.info('UFF optimizing those:')
  t1 = time.time()
  for mol in mols3d:
    if not mol.GetNumConformers():
      continue
    mol = Chem.Mol(mol)
    needMore = 1
    while needMore:
      needMore = AllChem.UFFOptimizeMolecule(mol, maxIters=200)
  t2 = time.time()
  logger.info('Results14: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[15]:
  logger.info('MMFF optimizing the molecules:')
  t1 = time.time()
  for i, mol in enumerate(mols3d):
    mol = Chem.Mol(mol)
    if not mol.GetNumConformers():
      continue
    if not AllChem.MMFFHasAllMoleculeParams(mol):
      continue
    needMore = 1
    while needMore:
      try:
        needMore = AllChem.MMFFOptimizeMolecule(mol, maxIters=200)
      except ValueError:
        logger.warning('Problems with MMFF and mol %d' % i)
        break
  t2 = time.time()
  logger.info('Results15: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[16]:
  logger.info('Find unique subgraphs')
  t1 = time.time()
  for mol in mols:
    Chem.FindUniqueSubgraphsOfLengthN(mol, 6)
  t2 = time.time()
  logger.info('Results16: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[17]:
  logger.info('Generate topological fingerprints')
  t1 = time.time()
  for mol in mols:
    Chem.RDKFingerprint(mol)
  t2 = time.time()
  logger.info('Results17: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

if tests[18]:
  logger.info('Generate morgan fingerprints')
  t1 = time.time()
  for mol in mols:
    AllChem.GetMorganFingerprint(mol, radius=2)
  t2 = time.time()
  logger.info('Results18: %.2f seconds' % (t2 - t1))
  ts.append(t2 - t1)

print(f"| {rdkit.__version__} | {' | '.join(['%.1f' % x for x in ts])} |")



import os.path
import pickle

from rdkit import Chem, RDConfig
from rdkit.Chem import Descriptors

descrs = [
  'BertzCT', 'Chi0', 'HallKierAlpha', 'Ipc', 'Kappa3', 'LabuteASA', 'SMR_VSA1', 'SMR_VSA10',
  'SMR_VSA2', 'SMR_VSA3', 'SMR_VSA4', 'SMR_VSA5', 'SMR_VSA6', 'SMR_VSA7', 'SMR_VSA8', 'SMR_VSA9',
  'SlogP_VSA1', 'SlogP_VSA10', 'SlogP_VSA11', 'SlogP_VSA12', 'SlogP_VSA2', 'SlogP_VSA3',
  'SlogP_VSA5', 'SlogP_VSA6', 'SlogP_VSA7', 'SlogP_VSA8', 'SlogP_VSA9', 'TPSA', 'NumHAcceptors',
  'NumHDonors', 'NumHeteroatoms', 'NumRotatableBonds', 'MolLogP'
]


def runIt(inFileName, outFileName, smiCol=0, maxMols=-1, delim=','):
  outF = open(outFileName, 'w+')
  outF.write(','.join(['SMILES'] + descrs))
  outF.write('\n')
  mols = []
  nDone = 0
  for line in inD:
    if line[0] != '#':
      splitL = line.strip().split(delim)
      if not splitL:
        continue
      smi = splitL[smiCol].strip()
      mol = Chem.MolFromSmiles(smi)
      print(smi)
      if mol:
        vals = []
        for descr in descrs:
          fn = getattr(Descriptors, descr)
          try:
            v = fn(mol)
          except Exception:
            v = 666
          vals.append(v)
        outF.write(','.join([smi] + ['%.4f' % x for x in vals]))
        outF.write('\n')
      nDone += 1
      if maxMols > 0 and nDone >= maxMols:
        break
  outF.close()


if __name__ == '__main__':
  if 1:
    # 100 PhysProp compounds
    inD = file(os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'PP_descrs_regress.2.csv'),
               'r').readlines()
    outFileName = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'PP_descrs_regress.2.csv')
    runIt(inD, outFileName, smiCol=0, delim=',')



import math
import os
import sys
import unittest

from rdkit import Chem, DataStructs, RDConfig
from rdkit.Chem import rdMolAlign
from rdkit.Chem import rdMolTransforms as rdmt
from rdkit.Chem import rdShapeHelpers as rdshp
from rdkit.Geometry import rdGeometry as geom


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1Shape(self):
    fileN = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'ShapeHelpers', 'test_data',
                         '1oir.mol')
    m = Chem.MolFromMolFile(fileN)
    rdmt.CanonicalizeMol(m)
    dims1, offset1 = rdshp.ComputeConfDimsAndOffset(m.GetConformer())
    grd = geom.UniformGrid3D(30.0, 16.0, 10.0)
    rdshp.EncodeShape(m, grd, 0)
    ovect = grd.GetOccupancyVect()
    self.assertEqual(ovect.GetTotalVal(), 7405)

    m = Chem.MolFromMolFile(fileN)
    trans = rdmt.ComputeCanonicalTransform(m.GetConformer())
    dims, offset = rdshp.ComputeConfDimsAndOffset(m.GetConformer(), trans=trans)
    dims -= dims1
    offset -= offset1
    self.assertAlmostEqual(dims.Length(), 0.0, 4)
    self.assertAlmostEqual(offset.Length(), 0.0, 4)

    grd1 = geom.UniformGrid3D(30.0, 16.0, 10.0)
    rdshp.EncodeShape(m, grd1, 0, trans)
    ovect = grd1.GetOccupancyVect()

    self.assertEqual(ovect.GetTotalVal(), 7405)

    grd2 = geom.UniformGrid3D(30.0, 16.0, 10.0)
    rdshp.EncodeShape(m, grd2, 0)

    fileN2 = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'ShapeHelpers', 'test_data',
                          '1oir_conf.mol')
    m2 = Chem.MolFromMolFile(fileN2)

    self.assertAlmostEqual(rdshp.ShapeTanimotoDist(m, m), 0.0, 4)
    self.assertAlmostEqual(rdshp.ShapeTverskyIndex(m, m, 1.0, 1.0), 1.0, 4)

    rmsd = rdMolAlign.AlignMol(m, m2)
    self.assertAlmostEqual(rdshp.ShapeTanimotoDist(m, m2), 0.31, 2)
    self.assertAlmostEqual(rdshp.ShapeTverskyIndex(m, m2, 1.0, 1.0), 0.686, 2)

    dist = rdshp.ShapeTanimotoDist(mol1=m, mol2=m2, confId1=0, confId2=0, gridSpacing=0.25,
                                   stepSize=0.125)
    self.assertAlmostEqual(dist, 0.339, 2)

    m = Chem.MolFromMolFile(fileN)
    cpt = rdmt.ComputeCentroid(m.GetConformer())
    dims, offset = rdshp.ComputeConfDimsAndOffset(m.GetConformer())

    grd = geom.UniformGrid3D(dims.x, dims.y, dims.z, 0.5, DataStructs.DiscreteValueType.TWOBITVALUE,
                             offset)
    dims -= geom.Point3D(13.927, 16.97, 9.775)
    offset -= geom.Point3D(-4.353, 16.829, 2.782)
    self.assertAlmostEqual(dims.Length(), 0.0, 4)
    self.assertAlmostEqual(offset.Length(), 0.0, 4)
    rdshp.EncodeShape(m, grd, 0)

    ovect = grd.GetOccupancyVect()

    self.assertEqual(ovect.GetTotalVal(), 7417)
    geom.WriteGridToFile(grd, '1oir_shape.grd')

    m = Chem.MolFromMolFile(fileN)
    lc, uc = rdshp.ComputeConfBox(m.GetConformer())
    rdmt.CanonicalizeMol(m)
    lc1, uc1 = rdshp.ComputeConfBox(m.GetConformer())

    lc2, uc2 = rdshp.ComputeUnionBox((lc, uc), (lc1, uc1))
    lc -= geom.Point3D(-4.353, 16.829, 2.782)
    uc -= geom.Point3D(9.574, 33.799, 12.557)
    self.assertAlmostEqual(lc.Length(), 0.0, 4)
    self.assertAlmostEqual(uc.Length(), 0.0, 4)

    lc1 -= geom.Point3D(-8.7163, -5.3279, -3.1621)
    uc1 -= geom.Point3D(10.7519, 6.0778, 3.0123)
    self.assertAlmostEqual(lc1.Length(), 0.0, 3)
    self.assertAlmostEqual(uc1.Length(), 0.0, 3)

    lc2 -= geom.Point3D(-8.716, -5.328, -3.162)
    uc2 -= geom.Point3D(10.752, 33.799, 12.557)
    self.assertAlmostEqual(lc2.Length(), 0.0, 3)
    self.assertAlmostEqual(uc2.Length(), 0.0, 3)


if __name__ == '__main__':
  print("Testing Shape Helpers wrapper")
  unittest.main()



import os
import unittest

from rdkit import Chem, DataStructs, Geometry, RDConfig
from rdkit.Chem import AllChem, ChemicalFeatures, rdDistGeom


def lstFeq(l1, l2, tol=1.e-4):
  if (len(list(l1)) != len(list(l2))):
    return 0
  for i in range(len(list(l1))):
    if not feq(l1[i], l2[i], tol):
      return 0
  return 1


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def testBasic(self):
    cfac = ChemicalFeatures.BuildFeatureFactory(
      os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolChemicalFeatures', 'test_data',
                   'featDef.txt'))
    self.assertTrue(cfac.GetNumFeatureDefs() == 2)

    fNames = cfac.GetFeatureFamilies()
    self.assertTrue(len(fNames) == 2)
    self.assertTrue(fNames[0] == 'HBondDonor')
    self.assertTrue(fNames[1] == 'HBondAcceptor')

    mol = Chem.MolFromSmiles("COCN")
    rdDistGeom.EmbedMolecule(mol, 30, 100, useExpTorsionAnglePrefs=False, useBasicKnowledge=False)

    self.assertTrue(cfac.GetNumMolFeatures(mol) == 3)
    for i in range(cfac.GetNumMolFeatures(mol)):
      self.assertTrue(cfac.GetMolFeature(mol, i))
    # check that the recompute argument works:
    self.assertTrue(cfac.GetMolFeature(mol, 0))
    for i in range(cfac.GetNumMolFeatures(mol)):
      self.assertTrue(cfac.GetMolFeature(mol, i, "", False))
    self.assertRaises(IndexError, lambda: cfac.GetMolFeature(mol, 3))

    feats = cfac.GetFeaturesForMol(mol)
    self.assertTrue(len(feats) == 3)
    fTypes = ['HBondDonor', 'HBondAcceptor', 'HBondAcceptor']

    positions = [[1.3041, -0.6079, 0.0924], [-0.7066, 0.5994, 0.1824], [1.3041, -0.6079, 0.0924]]
    targetAids = [[3], [1], [3]]
    for i, feat in enumerate(feats):
      self.assertEqual(feat.GetFamily(), fTypes[i])
      pos = list(feat.GetPos())
      aids = list(feat.GetAtomIds())
      self.assertEqual(aids, targetAids[i])
      self.assertTrue(lstFeq(pos, positions[i]))
      nmol = feat.GetMol()
      self.assertEqual(Chem.MolToSmiles(nmol), "COCN")
      ncfac = feat.GetFactory()
      self.assertEqual(ncfac.GetNumFeatureDefs(), 2)
      self.assertEqual(feat.GetActiveConformer(), -1)

  def testIncludeOnly(self):
    cfac = ChemicalFeatures.BuildFeatureFactory(
      os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'MolChemicalFeatures', 'test_data',
                   'featDef.txt'))
    self.assertTrue(cfac.GetNumFeatureDefs() == 2)

    mol = Chem.MolFromSmiles("COCN")
    rdDistGeom.EmbedMolecule(mol)

    self.assertTrue(cfac.GetNumMolFeatures(mol, includeOnly="HBondAcceptor") == 2)
    self.assertTrue(cfac.GetNumMolFeatures(mol, includeOnly="HBondDonor") == 1)
    self.assertTrue(cfac.GetNumMolFeatures(mol, includeOnly="Bogus") == 0)

    self.assertRaises(IndexError, lambda: cfac.GetMolFeature(mol, 1, includeOnly="HBondDonor"))
    self.assertRaises(IndexError,
                      lambda: cfac.GetMolFeature(mol, 2, includeOnly="HBondAcceptor"))
    f = cfac.GetMolFeature(mol, 0, includeOnly="HBondDonor")
    self.assertTrue(f.GetFamily() == 'HBondDonor')

    feats = cfac.GetFeaturesForMol(mol, includeOnly="HBondAcceptor")
    self.assertTrue(len(feats) == 2)
    feats = cfac.GetFeaturesForMol(mol, includeOnly="HBondDonor")
    self.assertTrue(len(feats) == 1)
    feats = cfac.GetFeaturesForMol(mol, includeOnly="Bogus")
    self.assertTrue(len(feats) == 0)

  def testStringParse(self):
    fdefBlock = \
"""DefineFeature HDonor1 [N,O;!H0]
    Family HBondDonor
    Weights 1.0
EndFeature
DefineFeature HAcceptor1 [N,O;H0]
    Family HBondAcceptor
    Weights 1.0
EndFeature
"""
    cfac = ChemicalFeatures.BuildFeatureFactoryFromString(fdefBlock)
    self.assertTrue(cfac.GetNumFeatureDefs() == 2)

  def testStringParse2(self):
    fdefBlock = \
"""DefineFeature HDonor1 [N,O;!H0]\r
    Family HBondDonor\r
    Weights 1.0\r
EndFeature\r
DefineFeature HAcceptor1 [N,O;H0]\r
    Family HBondAcceptor\r
    Weights 1.0\r
EndFeature\r
"""
    cfac = ChemicalFeatures.BuildFeatureFactoryFromString(fdefBlock)
    self.assertTrue(cfac.GetNumFeatureDefs() == 2)

  def testParseErrorHandling(self):
    fdefBlock = \
"""DefineFeature HDonor1 [N,O;!HQ]
    Family HBondDonor
    Weights 1.0
EndFeature
"""
    self.assertRaises(ValueError,
                      lambda: ChemicalFeatures.BuildFeatureFactoryFromString(fdefBlock))
    fdefBlock = \
"""DefineFeature HDonor1 [N,O;!H0]
    Family HBondDonor
    Weights 1.0
"""
    self.assertRaises(ValueError,
                      lambda: ChemicalFeatures.BuildFeatureFactoryFromString(fdefBlock))

    self.assertRaises(IOError, lambda: ChemicalFeatures.BuildFeatureFactory('noSuchFile.txt'))

  def testAtomMatch(self):
    fdefBlock = \
"""
DefineFeature HAcceptor1 [#7,#8]
    Family HBondAcceptor
    Weights 1.0
EndFeature
DefineFeature Arom1 a1aaaaa1
    Family Aromatic
    Weights 1.0,1.0,1.0,1.0,1.0,1.0
EndFeature
"""
    cfac = ChemicalFeatures.BuildFeatureFactoryFromString(fdefBlock)
    self.assertTrue(cfac.GetNumFeatureDefs() == 2)
    mol = Chem.MolFromSmiles('n1ccccc1')
    feats = cfac.GetFeaturesForMol(mol)
    self.assertTrue(len(feats) == 2)
    m = ChemicalFeatures.GetAtomMatch(feats)
    self.assertFalse(m)

    mol = Chem.MolFromSmiles('c1ccccc1N')
    feats = cfac.GetFeaturesForMol(mol)
    self.assertTrue(len(feats) == 2)
    m = ChemicalFeatures.GetAtomMatch(feats)
    self.assertTrue(len(m) == 2)

  def testIssue231(self):
    fdefs = """
DefineFeature HDonor1 [N,O;!H0]
  Family HBondDonor
  Weights 1.0
EndFeature
DefineFeature HAcceptor1 [N,O;H0]
  Family HBondAcceptor
  Weights 1.0
EndFeature
"""
    cfac = ChemicalFeatures.BuildFeatureFactoryFromString(fdefs)

    m = Chem.MolFromSmiles('O=CCCN')
    rdDistGeom.EmbedMolecule(m)
    feats = cfac.GetFeaturesForMol(m)
    for feat in feats:
      feat.GetPos()
    m = None
    for feat in feats:
      feat.GetPos()

  def testGithub2603(self):
    cfac = ChemicalFeatures.BuildFeatureFactory(
      os.path.join(RDConfig.RDDataDir, "BaseFeatures.fdef"))
    m = Chem.MolFromSmiles('OCc1ccccc1CN')
    feats = cfac.GetFeaturesForMol(m)
    self.assertEqual(feats[0].GetFamily(), 'Donor')
    cfac = None
    self.assertEqual(feats[0].GetFamily(), 'Donor')

  def testGithub2530(self):
    cfac = ChemicalFeatures.BuildFeatureFactory(
      os.path.join(RDConfig.RDDataDir, "BaseFeatures.fdef"))
    m = Chem.MolFromSmiles('C1CCC1OC')
    rdDistGeom.EmbedMolecule(m)
    feats = cfac.GetFeaturesForMol(m)
    feat_pos = feats[0].GetPos(-1)
    feat_pos_default = feats[0].GetPos()
    self.assertEqual(feat_pos[0], feat_pos_default[0])
    self.assertEqual(feat_pos[1], feat_pos_default[1])
    self.assertEqual(feat_pos[2], feat_pos_default[2])

    # Conformers generation:
    m2 = Chem.AddHs(m)
    AllChem.EmbedMultipleConfs(m2, numConfs=10, params=AllChem.ETKDG())

    feats_0 = cfac.GetFeaturesForMol(m2, confId=-1)
    feats_5 = cfac.GetFeaturesForMol(m2, confId=5)
    self.assertNotEqual(feats_5[0], feats_0[0])
    self.assertNotEqual(feats_5[1], feats_0[1])
    self.assertNotEqual(feats_5[2], feats_0[2])


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python

# Script to update the hash codes in the cpp codes.
# Takes 2 arguments, the output from running one of the test programs
# giving the values to update, and the corresponding cpp
# file.  Output is to {file.cpp}.update.
# Note that the PNG hash codes are only for Freetype builds, so it's
# better to do the non-Freetype builds first, otherwise the PNG
# hashes will definitely be wrong.

import json
import re
import sys

updates = {}
with open(sys.argv[1], 'r') as f:
  for line in f.readlines():
    if line.startswith('file'):
      line_bits = line.strip().split()
      updates[(line_bits[1], line_bits[8])] = (line_bits[1], line_bits[4])

re_str = r'{"([\w.-]*.\w*)", (\d*U)},'
patt = re.compile(re_str)

with open(sys.argv[2], 'r') as f:
  cpp_file = f.read()

hashes = patt.findall(cpp_file)


def hash_replace(match):
  match_bits = match.group().split(',')
  match_tuple = (match_bits[0][2:-1], match_bits[1][1:-1])
  if match_tuple in updates:
    rep_tuple = updates[match_tuple]
    rep_str = f'{{"{rep_tuple[0]}", {rep_tuple[1]}}},'
    print(f'replacing {match.group()} with {rep_str}')
    return rep_str
  else:
    return match.group()


for hash in hashes:
  print(hash)
  if hash in updates:
    print(hash, updates[hash])

new_cpp_file = patt.sub(hash_replace, cpp_file)

with open(f'{sys.argv[2]}.update', 'w') as f:
  f.write(new_cpp_file)



import doctest
import unittest

from rdkit import Chem
from rdkit.Chem import (BRICS, AllChem, Descriptors, EnumerateHeterocycles,
                        EnumerateStereoisomers, FragmentMatcher, MACCSkeys,
                        PropertyMol, Recap, SaltRemover, TemplateAlign,
                        rdDepictor)


def load_tests(loader, tests, ignore):  # pylint: disable=unused-argument
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(EnumerateStereoisomers, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(EnumerateHeterocycles, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(FragmentMatcher, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(MACCSkeys, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Descriptors, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Recap, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(BRICS, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(AllChem, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(PropertyMol, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SaltRemover, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Chem, optionflags=doctest.ELLIPSIS))
  # Tests which have a dependency on using the RDKit coordinate generator
  rdDepictor.SetPreferCoordGen(False)
  tests.addTests(doctest.DocTestSuite(TemplateAlign, optionflags=doctest.ELLIPSIS))

  return tests


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import sys
import unittest

from rdkit import Chem
from rdkit.Chem import rdFMCS


class BondMatchOrderMatrix:

  def __init__(self, ignoreAromatization):
    self.MatchMatrix = [[False] * (Chem.BondType.ZERO + 1) for i in range(Chem.BondType.ZERO + 1)]
    for i in range(Chem.BondType.ZERO + 1):
      # fill cells of the same and unspecified type
      self.MatchMatrix[i][i] = True
      self.MatchMatrix[Chem.BondType.UNSPECIFIED][i] = \
          self.MatchMatrix[i][Chem.BondType.UNSPECIFIED] = True
      self.MatchMatrix[Chem.BondType.ZERO][i] = \
          self.MatchMatrix[i][Chem.BondType.ZERO] = True
    if ignoreAromatization:
      self.MatchMatrix[Chem.BondType.SINGLE][Chem.BondType.AROMATIC] = \
          self.MatchMatrix[Chem.BondType.AROMATIC][Chem.BondType.SINGLE] = True
      self.MatchMatrix[Chem.BondType.DOUBLE][Chem.BondType.AROMATIC] = \
          self.MatchMatrix[Chem.BondType.AROMATIC][Chem.BondType.DOUBLE] = True
    self.MatchMatrix[Chem.BondType.SINGLE][Chem.BondType.ONEANDAHALF] = \
        self.MatchMatrix[Chem.BondType.ONEANDAHALF][Chem.BondType.SINGLE] = True
    self.MatchMatrix[Chem.BondType.DOUBLE][Chem.BondType.TWOANDAHALF] = \
        self.MatchMatrix[Chem.BondType.TWOANDAHALF][Chem.BondType.DOUBLE] = True
    self.MatchMatrix[Chem.BondType.TRIPLE][Chem.BondType.THREEANDAHALF] = \
        self.MatchMatrix[Chem.BondType.THREEANDAHALF][Chem.BondType.TRIPLE] = True
    self.MatchMatrix[Chem.BondType.QUADRUPLE][Chem.BondType.FOURANDAHALF] = \
        self.MatchMatrix[Chem.BondType.FOURANDAHALF][Chem.BondType.QUADRUPLE] = True
    self.MatchMatrix[Chem.BondType.QUINTUPLE][Chem.BondType.FIVEANDAHALF] = \
        self.MatchMatrix[Chem.BondType.FIVEANDAHALF][Chem.BondType.QUINTUPLE] = True

  def isEqual(self, i, j):
    return self.MatchMatrix[i][j]


class CompareAny(rdFMCS.MCSAtomCompare):

  def __call__(self, p, mol1, atom1, mol2, atom2):
    if (p.MatchChiralTag and not self.CheckAtomChirality(p, mol1, atom1, mol2, atom2)):
      return False
    if (p.MatchFormalCharge and not self.CheckAtomCharge(p, mol1, atom1, mol2, atom2)):
      return False
    if (p.RingMatchesRingOnly):
      return self.CheckAtomRingMatch(p, mol1, atom1, mol2, atom2)
    return True


class CompareAnyHeavyAtom(CompareAny):

  def __call__(self, p, mol1, atom1, mol2, atom2):
    a1 = mol1.GetAtomWithIdx(atom1)
    a2 = mol2.GetAtomWithIdx(atom2)
    # Any atom, including H, matches another atom of the same type,  according to
    # the other flags
    if (a1.GetAtomicNum() == a2.GetAtomicNum()
        or (a1.GetAtomicNum() > 1 and a2.GetAtomicNum() > 1)):
      return CompareAny.__call__(self, p, mol1, atom1, mol2, atom2)
    return False


class CompareElements(rdFMCS.MCSAtomCompare):

  def __call__(self, p, mol1, atom1, mol2, atom2):
    a1 = mol1.GetAtomWithIdx(atom1)
    a2 = mol2.GetAtomWithIdx(atom2)
    if (a1.GetAtomicNum() != a2.GetAtomicNum()):
      return False
    if (p.MatchValences and a1.GetTotalValence() != a2.GetTotalValence()):
      return False
    if (p.MatchChiralTag and not self.CheckAtomChirality(p, mol1, atom1, mol2, atom2)):
      return False
    if (p.MatchFormalCharge and not self.CheckAtomCharge(p, mol1, atom1, mol2, atom2)):
      return False
    if p.RingMatchesRingOnly:
      return self.CheckAtomRingMatch(p, mol1, atom1, mol2, atom2)
    return True


class CompareIsotopes(rdFMCS.MCSAtomCompare):

  def __call__(self, p, mol1, atom1, mol2, atom2):
    a1 = mol1.GetAtomWithIdx(atom1)
    a2 = mol2.GetAtomWithIdx(atom2)
    if (a1.GetIsotope() != a2.GetIsotope()):
      return False
    if (p.MatchChiralTag and not self.CheckAtomChirality(p, mol1, atom1, mol2, atom2)):
      return False
    if (p.MatchFormalCharge and not self.CheckAtomCharge(p, mol1, atom1, mol2, atom2)):
      return False
    if p.RingMatchesRingOnly:
      return self.CheckAtomRingMatch(p, mol1, atom1, mol2, atom2)
    return True


class CompareOrder(rdFMCS.MCSBondCompare):
  match = BondMatchOrderMatrix(True)  # ignore Aromatization

  def __call__(self, p, mol1, bond1, mol2, bond2):
    b1 = mol1.GetBondWithIdx(bond1)
    b2 = mol2.GetBondWithIdx(bond2)
    t1 = b1.GetBondType()
    t2 = b2.GetBondType()
    if self.match.isEqual(t1, t2):
      if (p.MatchStereo and not self.CheckBondStereo(p, mol1, bond1, mol2, bond2)):
        return False
      if p.RingMatchesRingOnly:
        return self.CheckBondRingMatch(p, mol1, bond1, mol2, bond2)
      return True
    return False


class AtomCompareCompareIsInt(rdFMCS.MCSAtomCompare):
  __call__ = 1


class AtomCompareNoCompare(rdFMCS.MCSAtomCompare):
  pass


class AtomCompareUserData(rdFMCS.MCSAtomCompare):

  def __init__(self):
    super().__init__()
    self._matchAnyHet = False

  def setMatchAnyHet(self, v):
    self._matchAnyHet = v

  def __call__(self, p, mol1, atom1, mol2, atom2):
    a1 = mol1.GetAtomWithIdx(atom1)
    a2 = mol2.GetAtomWithIdx(atom2)
    if (a1.GetAtomicNum() != a2.GetAtomicNum()
        and ((not self._matchAnyHet) or a1.GetAtomicNum() == 6 or a2.GetAtomicNum() == 6)):
      return False
    if (p.MatchValences and a1.GetTotalValence() != a2.GetTotalValence()):
      return False
    if (p.MatchChiralTag and not self.CheckAtomChirality(p, mol1, atom1, mol2, atom2)):
      return False
    if (p.MatchFormalCharge and not self.CheckAtomCharge(p, mol1, atom1, mol2, atom2)):
      return False
    if p.RingMatchesRingOnly:
      return self.CheckAtomRingMatch(p, mol1, atom1, mol2, atom2)
    return True


class BondCompareCompareIsInt(rdFMCS.MCSBondCompare):
  __call__ = 1


class BondCompareNoCompare(rdFMCS.MCSBondCompare):
  pass


class BondCompareUserData(rdFMCS.MCSBondCompare):

  def __init__(self):
    super().__init__()
    self.match = None

  def setIgnoreAromatization(self, v):
    self.match = BondMatchOrderMatrix(v)

  def __call__(self, p, mol1, bond1, mol2, bond2):
    b1 = mol1.GetBondWithIdx(bond1)
    b2 = mol2.GetBondWithIdx(bond2)
    t1 = b1.GetBondType()
    t2 = b2.GetBondType()
    if self.match.isEqual(t1, t2):
      if (p.MatchStereo and not self.CheckBondStereo(p, mol1, bond1, mol2, bond2)):
        return False
      if p.RingMatchesRingOnly:
        return self.CheckBondRingMatch(p, mol1, bond1, mol2, bond2)
      return True
    return False


class ProgressCallbackCallbackIsInt(rdFMCS.MCSProgress):
  __call__ = 1


class ProgressCallbackNoCallback(rdFMCS.MCSProgress):
  pass


class ProgressCallback(rdFMCS.MCSProgress):

  def __init__(self, parent):
    super().__init__()
    self.parent = parent
    self.callCount = 0

  def __call__(self, stat, params):
    self.callCount += 1
    self.parent.assertTrue(isinstance(stat, rdFMCS.MCSProgressData))
    self.parent.assertTrue(hasattr(stat, "numAtoms"))
    self.parent.assertTrue(isinstance(stat.numAtoms, int))
    self.parent.assertTrue(hasattr(stat, "numBonds"))
    self.parent.assertTrue(isinstance(stat.numBonds, int))
    self.parent.assertTrue(hasattr(stat, "seedProcessed"))
    self.parent.assertTrue(isinstance(stat.seedProcessed, int))
    self.parent.assertTrue(isinstance(params, rdFMCS.MCSParameters))
    self.parent.assertTrue(isinstance(params.AtomTyper, rdFMCS.MCSAtomCompare))
    self.parent.assertTrue(isinstance(params.BondTyper, rdFMCS.BondCompare))
    self.parent.assertEqual(params.ProgressCallback, self)
    return (self.callCount < 3)


class Test21AcceptanceCallbackCallbackIsInt(rdFMCS.MCSAcceptance):
  __call__ = 1


class Test21AcceptanceCallbackNoCallback(rdFMCS.MCSAcceptance):
  pass


class Test21AcceptanceCallback(rdFMCS.MCSAcceptance):

  def __init__(self, parent):
    super().__init__()
    self.parent = parent

  def __call__(self, query, target, atom_idx_match, bond_idx_match, params):
    self.parent.assertTrue(isinstance(query, Chem.Mol))
    self.parent.assertTrue(isinstance(target, Chem.Mol))
    self.parent.assertGreater(len(atom_idx_match), 0)
    self.parent.assertTrue(len(atom_idx_match) == 1 or len(bond_idx_match) > 0)
    self.parent.assertTrue(isinstance(params, rdFMCS.MCSParameters))
    self.parent.assertTrue(isinstance(params.AtomTyper, rdFMCS.MCSAtomCompare))
    self.parent.assertTrue(isinstance(params.BondTyper, rdFMCS.BondCompare))
    self.parent.assertEqual(params.ShouldAcceptMCS, self)
    for query_atom_idx, target_atom_idx in atom_idx_match:
      if query.GetAtomWithIdx(query_atom_idx).GetAtomicNum() == 0:
        return True 
      if target.GetAtomWithIdx(target_atom_idx).GetAtomicNum() == 0:
        return True
    for query_bond_idx, target_bond_idx in bond_idx_match:
      self.parent.assertIsNotNone(query.GetBondWithIdx(query_bond_idx))
      self.parent.assertIsNotNone(target.GetBondWithIdx(target_bond_idx))
    return False


class Test21AtomCompare(rdFMCS.MCSAtomCompare):

  def __init__(self):
    super().__init__()

  def __call__(self, p, mol1, atom1, mol2, atom2):
    a1 = mol1.GetAtomWithIdx(atom1)
    a2 = mol2.GetAtomWithIdx(atom2)
    a1_is_dummy = (a1.GetAtomicNum() == 0)
    a2_is_dummy = (a2.GetAtomicNum() == 0)
    if a1_is_dummy ^ a2_is_dummy:
      atoms = (a1, a2)
      dummy_atom_idx = 0 if a1_is_dummy else 1
      other_atom_idx = 1 - dummy_atom_idx
      return atoms[dummy_atom_idx].GetDegree() == 1 and atoms[other_atom_idx].GetDegree() > 1
    if a1.GetAtomicNum() != a2.GetAtomicNum():
      return False
    return self.CheckAtomRingMatch(p, mol1, atom1, mol2, atom2)


class Test21ProgressCallback(rdFMCS.MCSProgress):

  def __init__(self, parent):
    super().__init__()
    self.parent = parent
    self.callCount = 0

  def __call__(self, stat, params):
    self.callCount += 1
    self.parent.assertTrue(isinstance(stat, rdFMCS.MCSProgressData))
    self.parent.assertTrue(hasattr(stat, "numAtoms"))
    self.parent.assertTrue(isinstance(stat.numAtoms, int))
    self.parent.assertTrue(hasattr(stat, "numBonds"))
    self.parent.assertTrue(isinstance(stat.numBonds, int))
    self.parent.assertTrue(hasattr(stat, "seedProcessed"))
    self.parent.assertTrue(isinstance(stat.seedProcessed, int))
    self.parent.assertTrue(isinstance(params, rdFMCS.MCSParameters))
    self.parent.assertTrue(isinstance(params.AtomTyper, rdFMCS.MCSAtomCompare))
    self.parent.assertTrue(isinstance(params.BondTyper, rdFMCS.BondCompare))
    self.parent.assertEqual(params.ProgressCallback, self)
    return True


class Test21FinalMatchCheckCallbackCallbackIsInt(rdFMCS.MCSFinalMatchCheck):
  __call__ = 1


class Test21FinalMatchCheckCallbackNoCallback(rdFMCS.MCSFinalMatchCheck):
  pass


class Test21FinalMatchCheckCallback(rdFMCS.MCSFinalMatchCheck):

  def __init__(self, parent):
    super().__init__()
    self.parent = parent
    self.callCount = 0

  def __call__(self, mol1, mol2, atom_idx_match, bond_idx_match, params):
    self.callCount += 1
    self.parent.assertTrue(isinstance(mol1, Chem.Mol))
    self.parent.assertTrue(isinstance(mol2, Chem.Mol))
    self.parent.assertGreater(len(atom_idx_match), 0)
    self.parent.assertGreater(len(bond_idx_match), 0)
    self.parent.assertTrue(isinstance(params, rdFMCS.MCSParameters))
    self.parent.assertTrue(isinstance(params.AtomTyper, rdFMCS.MCSAtomCompare))
    self.parent.assertTrue(isinstance(params.BondTyper, rdFMCS.BondCompare))
    self.parent.assertEqual(params.FinalMatchChecker, self)
    return True


class Common:

  @staticmethod
  def getParams(**kwargs):
    params = rdFMCS.MCSParameters()
    for kw in ("AtomTyper", "BondTyper"):
      v = kwargs.get(kw, None)
      if v is not None:
        v_instance = v()
        setattr(params, kw, v_instance)
    return params

  @staticmethod
  def test1(self, **kwargs):
    smis = (
      "Cc1nc(CN(C(C)c2ncccc2)CCCCN)ccc1 CHEMBL1682991",  # -- QUERY
      "Cc1ccc(CN(C(C)c2ccccn2)CCCCN)nc1 CHEMBL1682990",
      "Cc1cccnc1CN(C(C)c1ccccn1)CCCCN CHEMBL1682998",
      "CC(N(CCCCN)Cc1c(N)cccn1)c1ccccn1 CHEMBL1682987",
      "Cc1cc(C)c(CN(C(C)c2ccccn2)CCCCN)nc1 CHEMBL1682992",
      "Cc1cc(C(C)N(CCCCN)Cc2c(C)cccn2)ncc1 CHEMBL1682993",
      "Cc1nc(C(C)N(CCCCN)Cc2nc3c([nH]2)cccc3)ccc1 CHEMBL1682878",
      "CC(c1ncccc1)N(CCCCN)Cc1nc2c([nH]1)cccc2 CHEMBL1682867",
      "CC(N(CCCCN)Cc1c(C(C)(C)C)cccn1)c1ccccn1 CHEMBL1682989",
      "CC(N(CCCCN)Cc1c(C(F)(F)F)cccn1)c1ccccn1 CHEMBL1682988",
    )
    ms = [Chem.MolFromSmiles(x.split()[0]) for x in smis]
    qm = ms[0]
    ms = ms[1:]
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 21)
    self.assertEqual(mcs.numAtoms, 21)
    self.assertEqual(
      mcs.smartsString,
      '[#6](:[#6]:[#6]):[#6]:[#7]:[#6]-[#6]-[#7](-[#6](-[#6])-[#6]1:[#6]:[#6]:[#6]:[#6]:[#7]:1)-[#6]-[#6]-[#6]-[#6]-[#7]'
    )
    qm = Chem.MolFromSmarts(mcs.smartsString)
    self.assertTrue(qm is not None)
    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

    if kwargs:
      params = Common.getParams(**kwargs)
      params.Threshold = 0.8
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, threshold=0.8)
    self.assertEqual(mcs.numBonds, 21)
    self.assertEqual(mcs.numAtoms, 21)
    self.assertEqual(
      mcs.smartsString,
      '[#6](:[#6]:[#6]):[#6]:[#7]:[#6]-[#6]-[#7](-[#6](-[#6])-[#6]1:[#6]:[#6]:[#6]:[#6]:[#7]:1)-[#6]-[#6]-[#6]-[#6]-[#7]'
    )
    qm = Chem.MolFromSmarts(mcs.smartsString)
    self.assertTrue(qm is not None)
    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

  def test2(self, **kwargs):
    smis = (
      "CHEMBL122452 CN(CCCN(C)CCc1ccccc1)CCOC(c1ccccc1)c1ccccc1",
      "CHEMBL123252 CN(CCCc1ccccc1)CCCN(C)CCOC(c1ccccc1)c1ccccc1",
      "CHEMBL121611 Fc1ccc(C(OCCNCCCNCCc2ccccc2)c2ccc(F)cc2)cc1",
      "CHEMBL121050 O=C(Cc1ccccc1)NCCCCNCCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL333667 O=C(Cc1ccccc1)NCCNCCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL121486 O=C(Cc1ccc(Br)cc1)NC=CNCCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL123830 O=C(Cc1ccc(F)cc1)NCCNCCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL420900 O=C(Cc1ccccc1)NCCCNCCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL121460 CN(CCOC(c1ccc(F)cc1)c1ccc(F)cc1)CCN(C)CCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL120901 COC(=O)C1C2CCC(CC1C(=O)Oc1ccccc1)N2C",
      "CHEMBL122859 O=C1CN(CCc2ccccc2)CCN1CCOC(c1ccc(F)cc1)c1ccc(F)cc1",
      "CHEMBL121027 CN(CCOC(c1ccccc1)c1ccccc1)CCN(C)CCc1ccc(F)cc1",
    )

    ms = [Chem.MolFromSmiles(x.split()[1]) for x in smis]
    qm = ms[0]
    ms = ms[1:]
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 9)
    self.assertEqual(mcs.numAtoms, 10)
    qm = Chem.MolFromSmarts(mcs.smartsString)
    self.assertTrue(qm is not None)
    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))
    # smarts too hard to canonicalize this
    # self.assertEqual(mcs.smartsString,'[#6]-,:[#6]-,:[#6]-,:[#6]-,:[#6](-[#6]-[#8]-[#6]:,-[#6])-,:[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.Threshold = 0.8
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, threshold=0.8)
    self.assertEqual(mcs.numBonds, 20)
    self.assertEqual(mcs.numAtoms, 19)
    qm = Chem.MolFromSmarts(mcs.smartsString)
    self.assertTrue(qm is not None)
    nHits = 0
    for m in ms:
      if m.HasSubstructMatch(qm):
        nHits += 1
    self.assertTrue(nHits >= int(0.8 * len(smis)))
    # smarts too hard to canonicalize this
    # self.assertEqual(mcs.smartsString,'[#6]1:[#6]:[#6]:[#6](:[#6]:[#6]:1)-[#6](-[#8]-[#6]-[#6]-[#7]-[#6]-[#6])-[#6]2:[#6]:[#6]:[#6]:[#6]:[#6]:2')

  def test3IsotopeMatch(self, **kwargs):
    smis = (
      "CC[14NH2]",
      "CC[14CH3]",
    )

    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomTyper = CompareIsotopes()
      params.AtomCompareParameters.MatchIsotope = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, atomCompare=rdFMCS.AtomCompare.CompareIsotopes)
    self.assertEqual(mcs.numBonds, 2)
    self.assertEqual(mcs.numAtoms, 3)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    self.assertTrue(Chem.MolFromSmiles('CC[14CH3]').HasSubstructMatch(qm))
    self.assertFalse(Chem.MolFromSmiles('CC[13CH3]').HasSubstructMatch(qm))
    self.assertTrue(Chem.MolFromSmiles('OO[14CH3]').HasSubstructMatch(qm))
    self.assertFalse(Chem.MolFromSmiles('O[13CH2][14CH3]').HasSubstructMatch(qm))

  def test4RingMatches(self, **kwargs):
    smis = ['CCCCC', 'CCC1CCCCC1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 4)
    self.assertEqual(mcs.numAtoms, 5)
    self.assertEqual(mcs.smartsString, '[#6]-[#6]-[#6]-[#6]-[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.BondCompareParameters.CompleteRingsOnly = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, completeRingsOnly=True)
    self.assertEqual(mcs.numBonds, 2)
    self.assertEqual(mcs.numAtoms, 3)
    self.assertEqual(mcs.smartsString, '[#6]-&!@[#6]-&!@[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.BondCompareParameters.CompleteRingsOnly = True
      params.BondCompareParameters.MatchFusedRings = True
      params.BondCompareParameters.MatchFusedRingsStrict = False
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, completeRingsOnly=True,
                           ringCompare=rdFMCS.RingCompare.PermissiveRingFusion)
    self.assertEqual(mcs.numBonds, 2)
    self.assertEqual(mcs.numAtoms, 3)
    self.assertEqual(mcs.smartsString, '[#6]-&!@[#6]-&!@[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.BondCompareParameters.CompleteRingsOnly = True
      params.BondCompareParameters.MatchFusedRings = True
      params.BondCompareParameters.MatchFusedRingsStrict = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, completeRingsOnly=True,
                           ringCompare=rdFMCS.RingCompare.StrictRingFusion)
    self.assertEqual(mcs.numBonds, 2)
    self.assertEqual(mcs.numAtoms, 3)
    self.assertEqual(mcs.smartsString, '[#6]-&!@[#6]-&!@[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    self.assertEqual(mcs.smartsString, '[#6&!R]-&!@[#6&!R]')

    smis = ['CC1CCC1', 'CCC1CCCCC1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 4)
    self.assertEqual(mcs.numAtoms, 5)
    self.assertEqual(mcs.smartsString, '[#6]-[#6](-[#6]-[#6])-[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.BondCompareParameters.CompleteRingsOnly = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, completeRingsOnly=True)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    self.assertEqual(mcs.smartsString, '[#6]-&!@[#6]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.CompleteRingsOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True, completeRingsOnly=True)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    self.assertEqual(mcs.smartsString, '[#6&!R]-&!@[#6&R]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.CompleteRingsOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.MatchFusedRings = True
      params.BondCompareParameters.MatchFusedRingsStrict = False
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True, completeRingsOnly=True,
                           ringCompare=rdFMCS.RingCompare.PermissiveRingFusion)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    self.assertEqual(mcs.smartsString, '[#6&!R]-&!@[#6&R]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.CompleteRingsOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.MatchFusedRings = True
      params.BondCompareParameters.MatchFusedRingsStrict = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True, completeRingsOnly=True,
                           ringCompare=rdFMCS.RingCompare.StrictRingFusion)
    self.assertEqual(mcs.numBonds, 1)
    self.assertEqual(mcs.numAtoms, 2)
    self.assertEqual(mcs.smartsString, '[#6&!R]-&!@[#6&R]')

    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True)
    self.assertEqual(mcs.numBonds, 4)
    self.assertEqual(mcs.numAtoms, 5)
    self.assertEqual(mcs.smartsString, '[#6&!R]-&!@[#6&R](-&@[#6&R]-&@[#6&R])-&@[#6&R]')

  def test5AnyMatch(self, **kwargs):
    smis = ('c1ccccc1C', 'c1ccccc1O', 'c1ccccc1Cl')
    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomTyper = CompareAny()
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, atomCompare=rdFMCS.AtomCompare.CompareAny)
    self.assertEqual(mcs.numBonds, 7)
    self.assertEqual(mcs.numAtoms, 7)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

    smis = ('c1cccnc1C', 'c1cnncc1O', 'c1cccnc1Cl')
    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomTyper = CompareAny()
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, atomCompare=rdFMCS.AtomCompare.CompareAny)
    self.assertEqual(mcs.numBonds, 7)
    self.assertEqual(mcs.numAtoms, 7)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

  def testAtomCompareAnyHeavyAtom(self, **kwargs):
    # H matches H, O matches C
    smis = ('[H]c1ccccc1C', '[H]c1ccccc1O')
    ms = [Chem.MolFromSmiles(x, sanitize=False) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomTyper = CompareAnyHeavyAtom()
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, atomCompare=rdFMCS.AtomCompare.CompareAnyHeavyAtom)
    self.assertEqual(mcs.numBonds, 8)
    self.assertEqual(mcs.numAtoms, 8)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

  def testAtomCompareAnyHeavyAtom1(self, **kwargs):
    # O matches C, H does not match O
    smis = ('[H]c1ccccc1C', 'Oc1ccccc1O')
    ms = [Chem.MolFromSmiles(x, sanitize=False) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomTyper = CompareAnyHeavyAtom()
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, atomCompare=rdFMCS.AtomCompare.CompareAnyHeavyAtom)
    self.assertEqual(mcs.numBonds, 7)
    self.assertEqual(mcs.numAtoms, 7)
    qm = Chem.MolFromSmarts(mcs.smartsString)

    for m in ms:
      self.assertTrue(m.HasSubstructMatch(qm))

  def test6MatchValences(self, **kwargs):
    ms = (Chem.MolFromSmiles('NC1OC1'), Chem.MolFromSmiles('C1OC1[N+](=O)[O-]'))
    if kwargs:
      params = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numBonds, 4)
    self.assertEqual(mcs.numAtoms, 4)
    if kwargs:
      params = Common.getParams(**kwargs)
      params.AtomCompareParameters.MatchValences = True
      mcs = rdFMCS.FindMCS(ms, params)
    else:
      mcs = rdFMCS.FindMCS(ms, matchValences=True)
    self.assertEqual(mcs.numBonds, 3)
    self.assertEqual(mcs.numAtoms, 3)

  def test7Seed(self, **kwargs):
    smis = ['C1CCC1CC1CC1', 'C1CCC1OC1CC1', 'C1CCC1NC1CC1', 'C1CCC1SC1CC1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    if kwargs:
      params = Common.getParams(**kwargs)
      r = rdFMCS.FindMCS(ms, params)
    else:
      r = rdFMCS.FindMCS(ms)
    self.assertEqual(r.smartsString, "[#6]1-[#6]-[#6]-[#6]-1")
    if kwargs:
      params = Common.getParams(**kwargs)
      params.InitialSeed = 'C1CC1'
      r = rdFMCS.FindMCS(ms, params)
    else:
      r = rdFMCS.FindMCS(ms, seedSmarts='C1CC1')
    self.assertEqual(r.smartsString, "[#6]1-[#6]-[#6]-1")
    if kwargs:
      params = Common.getParams(**kwargs)
      params.InitialSeed = 'C1OC1'
      r = rdFMCS.FindMCS(ms, params)
    else:
      r = rdFMCS.FindMCS(ms, seedSmarts='C1OC1')
    self.assertEqual(r.smartsString, "[#6]1-[#6]-[#6]-[#6]-1")
    self.assertEqual(r.numAtoms, 4)
    self.assertEqual(r.numBonds, 4)
    if kwargs:
      params = Common.getParams(**kwargs)
      params.InitialSeed = 'C1OC1'
      params.AtomCompareParameters.RingMatchesRingOnly = True
      params.BondCompareParameters.RingMatchesRingOnly = True
      r = rdFMCS.FindMCS(ms, params)
    else:
      r = rdFMCS.FindMCS(ms, seedSmarts='C1OC1', ringMatchesRingOnly=True)
    self.assertEqual(r.smartsString, "[#6]1-&@[#6]-&@[#6]-&@[#6]-&@1")
    self.assertEqual(r.numAtoms, 4)
    self.assertEqual(r.numBonds, 4)
    if kwargs:
      params = Common.getParams(**kwargs)
      params.InitialSeed = 'C1OC1'
      params.BondCompareParameters.CompleteRingsOnly = True
      r = rdFMCS.FindMCS(ms, params)
    else:
      r = rdFMCS.FindMCS(ms, seedSmarts='C1OC1', completeRingsOnly=True)
    self.assertEqual(r.smartsString, "[#6]1-&@[#6]-&@[#6]-&@[#6]-&@1")
    self.assertEqual(r.numAtoms, 4)
    self.assertEqual(r.numBonds, 4)

  def test8MatchParams(self, **kwargs):
    smis = ("CCC1NC1", "CCC1N(C)C1", "CCC1OC1")
    ms = [Chem.MolFromSmiles(x) for x in smis]

    if kwargs:
      ps = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, ps)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numAtoms, 4)

    ps = Common.getParams(**kwargs)
    ps.BondCompareParameters.CompleteRingsOnly = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 3)

    ps = Common.getParams(**kwargs)
    ps.BondCompareParameters.CompleteRingsOnly = True
    ps.BondCompareParameters.MatchFusedRings = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 3)

    ps = Common.getParams(**kwargs)
    ps.BondCompareParameters.CompleteRingsOnly = True
    ps.BondCompareParameters.MatchFusedRingsStrict = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 3)

    ps = Common.getParams(**kwargs)
    if kwargs:
      ps.AtomTyper = CompareAny()
    else:
      ps.AtomTyper = rdFMCS.AtomCompare.CompareAny
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 5)

  def test9MatchCharge(self, **kwargs):
    smis = ("CCNC", "CCN(C)C", "CC[N+](C)C")
    ms = [Chem.MolFromSmiles(x) for x in smis]

    if kwargs:
      ps = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, ps)
    else:
      mcs = rdFMCS.FindMCS(ms)

    self.assertEqual(mcs.numAtoms, 4)

    ps = Common.getParams(**kwargs)
    ps.AtomCompareParameters.MatchFormalCharge = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 2)

  def test10MatchChargeAndParams(self, **kwargs):
    smis = ("CCNC", "CCN(C)C", "CC[N+](C)C", "CC[C+](C)C")
    ms = [Chem.MolFromSmiles(x) for x in smis]

    if kwargs:
      ps = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, ps)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numAtoms, 2)

    ps = Common.getParams(**kwargs)
    if kwargs:
      ps.AtomTyper = CompareAny()
    else:
      ps.AtomTyper = rdFMCS.AtomCompare.CompareAny
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 4)

    ps = Common.getParams(**kwargs)
    ps.AtomCompareParameters.MatchFormalCharge = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 2)

  def test11Github2034(self, **kwargs):
    smis = ("C1CC1N2CC2", "C1CC1N")
    ms = [Chem.MolFromSmiles(x) for x in smis]

    if kwargs:
      ps = Common.getParams(**kwargs)
      mcs = rdFMCS.FindMCS(ms, ps)
    else:
      mcs = rdFMCS.FindMCS(ms)
    self.assertEqual(mcs.numAtoms, 4)
    self.assertEqual(mcs.numBonds, 4)

    if kwargs:
      ps = Common.getParams(**kwargs)
      ps.AtomCompareParameters.RingMatchesRingOnly = True
      ps.BondCompareParameters.RingMatchesRingOnly = True
      mcs = rdFMCS.FindMCS(ms, ps)
    else:
      mcs = rdFMCS.FindMCS(ms, ringMatchesRingOnly=True)
    self.assertEqual(mcs.numAtoms, 3)
    self.assertEqual(mcs.numBonds, 3)

    ps = Common.getParams(**kwargs)
    ps.AtomCompareParameters.RingMatchesRingOnly = True
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 3)
    self.assertEqual(mcs.numBonds, 3)

  def test19MCS3d(self, **kwargs):
    block1 = """
     RDKit          3D

 17 17  0  0  0  0  0  0  0  0999 V2000
    0.1592    0.8577    0.8639 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.9090    0.9385   -0.2218 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.1866   -0.4482   -0.7339 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.0705   -1.1345   -1.1348 O   0  0  0  0  0  0  0  0  0  0  0  0
    0.8460   -1.2511   -0.0884 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.3508    0.1733    0.1788 C   0  0  1  0  0  0  0  0  0  0  0  0
    1.6294    0.7269   -1.0491 O   0  0  0  0  0  0  0  0  0  0  0  0
   -0.2088    0.1739    1.6399 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.4334    1.8588    1.2198 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.8657    1.3295    0.1967 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.5185    1.5822   -1.0340 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.7155   -1.0290    0.0396 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.8809   -0.3833   -1.6049 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.6828   -1.9159   -0.3875 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.3276   -1.6370    0.8016 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.2158    0.0912    0.8546 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.6002    1.6926   -1.1014 H   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0
  2  3  1  0
  3  4  1  0
  4  5  1  0
  5  6  1  0
  6  7  1  0
  6  1  1  0
  1  8  1  0
  1  9  1  0
  2 10  1  0
  2 11  1  0
  3 12  1  0
  3 13  1  0
  5 14  1  0
  5 15  1  0
  6 16  1  1
  7 17  1  0
M  END


"""
    block2 = """
     RDKit          3D

 17 17  0  0  0  0  0  0  0  0999 V2000
    0.1592    0.8577    0.8639 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.9090    0.9385   -0.2218 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.1866   -0.4482   -0.7339 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.0705   -1.1345   -1.1348 O   0  0  0  0  0  0  0  0  0  0  0  0
    0.8460   -1.2511   -0.0884 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.3508    0.1733    0.1788 C   0  0  2  0  0  0  0  0  0  0  0  0
    2.4771    0.1924    0.9509 O   0  0  0  0  0  0  0  0  0  0  0  0
   -0.1638    0.2755    1.7318 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.4849    1.8825    1.0902 H   0  0  0  0  0  0  0  0  0  0  0  0
   -0.4598    1.5686   -1.0290 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.8337    1.3595    0.1902 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.8991   -0.3820   -1.5700 H   0  0  0  0  0  0  0  0  0  0  0  0
   -1.6394   -1.0265    0.1178 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.6816   -1.9167   -0.3664 H   0  0  0  0  0  0  0  0  0  0  0  0
    0.3727   -1.5979    0.8445 H   0  0  0  0  0  0  0  0  0  0  0  0
    1.4834    0.6899   -0.7827 H   0  0  0  0  0  0  0  0  0  0  0  0
    2.2370    0.0336    1.8861 H   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0
  2  3  1  0
  3  4  1  0
  4  5  1  0
  5  6  1  0
  6  7  1  0
  6  1  1  0
  1  8  1  0
  1  9  1  0
  2 10  1  0
  2 11  1  0
  3 12  1  0
  3 13  1  0
  5 14  1  0
  5 15  1  0
  6 16  1  6
  7 17  1  0
M  END


"""
    m1 = Chem.MolFromMolBlock(block1, removeHs=False)
    m2 = Chem.MolFromMolBlock(block2, removeHs=False)
    ps = Common.getParams(**kwargs)
    ps.AtomCompareParameters.MaxDistance = 1.0
    mcs = rdFMCS.FindMCS([m1, m2], ps)
    self.assertEqual(mcs.numAtoms, 14)
    self.assertEqual(mcs.numBonds, 14)


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1(self):
    Common.test1(self)

  def test1PythonImpl(self):
    Common.test1(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test2(self):
    Common.test2(self)

  def test2PythonImpl(self):
    Common.test2(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test2PythonImplAtomTyperOnly(self):
    Common.test2(self, AtomTyper=CompareElements)

  def test2PythonImplBondTyperOnly(self):
    Common.test2(self, BondTyper=CompareOrder)

  def test3IsotopeMatch(self):
    Common.test3IsotopeMatch(self)

  def test3IsotopeMatchPythonImpl(self):
    Common.test3IsotopeMatch(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test3IsotopeMatchPythonImplAtomTyperOnly(self):
    Common.test3IsotopeMatch(self, AtomTyper=CompareElements)

  def test3IsotopeMatchPythonImplBondTyperOnly(self):
    Common.test3IsotopeMatch(self, BondTyper=CompareOrder)

  def test4RingMatches(self):
    Common.test4RingMatches(self)

  def test4RingMatchesPythonImpl(self):
    Common.test4RingMatches(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test4RingMatchesPythonImplAtomTyperOnly(self):
    Common.test4RingMatches(self, AtomTyper=CompareElements)

  def test4RingMatchesPythonImplBondTyperOnly(self):
    Common.test4RingMatches(self, BondTyper=CompareOrder)

  def test5AnyMatch(self):
    Common.test5AnyMatch(self)

  def test5AnyMatchPythonImpl(self):
    Common.test5AnyMatch(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test5AnyMatchPythonImplAtomTyperOnly(self):
    Common.test5AnyMatch(self, AtomTyper=CompareElements)

  def test5AnyMatchPythonImplBondTyperOnly(self):
    Common.test5AnyMatch(self, BondTyper=CompareOrder)

  def testAtomCompareAnyHeavyAtom(self):
    Common.testAtomCompareAnyHeavyAtom(self)

  def testAtomCompareAnyHeavyAtomPythonImpl(self):
    Common.testAtomCompareAnyHeavyAtom(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def testAtomCompareAnyHeavyAtomPythonImplAtomTyperOnly(self):
    Common.testAtomCompareAnyHeavyAtom(self, AtomTyper=CompareElements)

  def testAtomCompareAnyHeavyAtomPythonImplBondTyperOnly(self):
    Common.testAtomCompareAnyHeavyAtom(self, BondTyper=CompareOrder)

  def testAtomCompareAnyHeavyAtom1(self):
    Common.testAtomCompareAnyHeavyAtom1(self)

  def testAtomCompareAnyHeavyAtom1PythonImpl(self):
    Common.testAtomCompareAnyHeavyAtom1(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def testAtomCompareAnyHeavyAtom1PythonImplAtomTyperOnly(self):
    Common.testAtomCompareAnyHeavyAtom1(self, AtomTyper=CompareElements)

  def testAtomCompareAnyHeavyAtom1PythonImplBondTyperOnly(self):
    Common.testAtomCompareAnyHeavyAtom1(self, BondTyper=CompareOrder)

  def test6MatchValences(self):
    Common.test6MatchValences(self)

  def test6MatchValencesPythonImpl(self):
    Common.test6MatchValences(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test6MatchValencesPythonImplAtomTyperOnly(self):
    Common.test6MatchValences(self, AtomTyper=CompareElements)

  def test6MatchValencesPythonImplBondTyperOnly(self):
    Common.test6MatchValences(self, BondTyper=CompareOrder)

  def test7Seed(self):
    Common.test7Seed(self)

  def test7SeedPythonImpl(self):
    Common.test7Seed(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test7SeedPythonImplAtomTyperOnly(self):
    Common.test7Seed(self, AtomTyper=CompareElements)

  def test7SeedPythonImplBondTyperOnly(self):
    Common.test7Seed(self, BondTyper=CompareOrder)

  def test8MatchParams(self):
    Common.test8MatchParams(self)

  def test8MatchParamsPythonImpl(self):
    Common.test8MatchParams(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test8MatchParamsPythonImplAtomTyperOnly(self):
    Common.test8MatchParams(self, AtomTyper=CompareElements)

  def test8MatchParamsPythonImplBondTyperOnly(self):
    Common.test8MatchParams(self, BondTyper=CompareOrder)

  def test9MatchCharge(self):
    Common.test9MatchCharge(self)

  def test9MatchChargePythonImpl(self):
    Common.test9MatchCharge(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test9MatchChargePythonImplAtomTyperOnly(self):
    Common.test9MatchCharge(self, AtomTyper=CompareElements)

  def test9MatchChargePythonImplBondTyperOnly(self):
    Common.test9MatchCharge(self, BondTyper=CompareOrder)

  def test10MatchChargeAndParams(self):
    Common.test10MatchChargeAndParams(self)

  def test10MatchChargeAndParamsPythonImpl(self):
    Common.test10MatchChargeAndParams(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test10MatchChargeAndParamsPythonImplAtomTyperOnly(self):
    Common.test10MatchChargeAndParams(self, AtomTyper=CompareElements)

  def test10MatchChargeAndParamsPythonImplBondTyperOnly(self):
    Common.test10MatchChargeAndParams(self, BondTyper=CompareOrder)

  def test11Github2034(self):
    Common.test11Github2034(self)

  def test11Github2034PythonImpl(self):
    Common.test11Github2034(self, AtomTyper=CompareElements, BondTyper=CompareOrder)

  def test11Github2034PythonImplAtomTyperOnly(self):
    Common.test11Github2034(self, AtomTyper=CompareElements)

  def test11Github2034PythonImplBondTyperOnly(self):
    Common.test11Github2034(self, BondTyper=CompareOrder)

  def test12MCSAtomCompareExceptions(self):
    ps = rdFMCS.MCSParameters()
    smis = ['CCCCC', 'CCC1CCCCC1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    self.assertRaises(TypeError, lambda ps: setattr(ps, "AtomTyper", AtomCompareCompareIsInt()))
    self.assertRaises(TypeError, lambda ps: setattr(ps, "AtomTyper", AtomCompareNoCompare()))

  def test13MCSAtomCompareUserData(self):
    smis = ['CCOCCOC', 'CCNCCCC']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    ps = rdFMCS.MCSParameters()
    ps.AtomTyper = AtomCompareUserData()
    ps.AtomTyper.setMatchAnyHet(False)
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 2)
    ps.AtomTyper.setMatchAnyHet(True)
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 5)

  def test14MCSBondCompareExceptions(self):
    ps = rdFMCS.MCSParameters()
    smis = ['CCCCC', 'CCC1CCCCC1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    self.assertRaises(TypeError, lambda ps: setattr(ps, "BondTyper", BondCompareCompareIsInt()))
    self.assertRaises(TypeError, lambda ps: setattr(ps, "BondTyper", BondCompareNoCompare()))

  def test15MCSBondCompareUserData(self):
    smis = ['C1CC=CCC1', 'c1ccccc1']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    ps = rdFMCS.MCSParameters()
    ps.BondTyper = BondCompareUserData()
    ps.BondCompareParameters.CompleteRingsOnly = True
    ps.BondTyper.setIgnoreAromatization(False)
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 0)
    ps.BondTyper.setIgnoreAromatization(True)
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertEqual(mcs.numAtoms, 6)

  def test16MCSProgressCallbackExceptions(self):
    ps = rdFMCS.MCSParameters()
    smis = ['CCCC(C)CC(CC)CC', 'OC(N)CC(C)CC(CC)CC']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    self.assertRaises(TypeError,
                      lambda ps: setattr(ps, "ProgressCallback", ProgressCallbackCallbackIsInt()))
    self.assertRaises(TypeError,
                      lambda ps: setattr(ps, "ProgressCallback", ProgressCallbackNoCallback()))

  def test17MCSProgressCallbackCancel(self):
    ps = rdFMCS.MCSParameters()
    smis = ['CCCC(C)CC(CC)CC', 'OC(N)CC(C)CC(CC)CC']
    ms = [Chem.MolFromSmiles(x) for x in smis]
    ps.AtomTyper = CompareElements()
    ps.ProgressCallback = ProgressCallback(self)
    mcs = rdFMCS.FindMCS(ms, ps)
    self.assertTrue(mcs.canceled)
    self.assertEqual(ps.ProgressCallback.callCount, 3)

  def test18GitHub3693(self):
    mols = [
      Chem.MolFromSmiles(smi)
      for smi in ["Nc1ccc(O)cc1c1ccc2ccccc2c1", "Oc1cnc(NC2CCC2)c(c1)c1ccc2ccccc2c1"]
    ]
    params = rdFMCS.MCSParameters()
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 17)
    self.assertEqual(res.numBonds, 18)
    self.assertEqual(
      res.smartsString,
      "[#7]-,:[#6]:[#6](:[#6]:[#6](:[#6])-[#8])-[#6]1:[#6]:[#6]:[#6]2:[#6](:[#6]:1):[#6]:[#6]:[#6]:[#6]:2"
    )

    params = rdFMCS.MCSParameters()
    params.BondCompareParameters.CompleteRingsOnly = True
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 11)
    self.assertEqual(res.numBonds, 12)
    self.assertEqual(
      res.smartsString,
      "[#6]-&!@[#6]1:&@[#6]:&@[#6]:&@[#6]2:&@[#6](:&@[#6]:&@1):&@[#6]:&@[#6]:&@[#6]:&@[#6]:&@2")

    params = rdFMCS.MCSParameters()
    params.AtomCompareParameters.CompleteRingsOnly = True
    params.BondCompareParameters.CompleteRingsOnly = True
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 10)
    self.assertEqual(res.numBonds, 11)
    self.assertEqual(
      res.smartsString,
      "[#6]1:&@[#6]:&@[#6]:&@[#6]2:&@[#6](:&@[#6]:&@1):&@[#6]:&@[#6]:&@[#6]:&@[#6]:&@2"
    )

    params = rdFMCS.MCSParameters()
    params.AtomCompareParameters.CompleteRingsOnly = True
    # this will automatically be set to True
    params.BondCompareParameters.CompleteRingsOnly = False
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 10)
    self.assertEqual(res.numBonds, 11)
    self.assertEqual(
      res.smartsString,
      "[#6]1:&@[#6]:&@[#6]:&@[#6]2:&@[#6](:&@[#6]:&@1):&@[#6]:&@[#6]:&@[#6]:&@[#6]:&@2"
    )

  def test19MCS3d(self):
    Common.test19MCS3d(self)

  def Test21AtomCompareCompleteRingsOnly(self):
    mols = [Chem.MolFromSmiles(smi) for smi in ["C1CCCC1C", "C1CCCC1C1CCCCC1"]]
    params = rdFMCS.MCSParameters()
    params.AtomCompareParameters.CompleteRingsOnly = True
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 5)
    self.assertEqual(res.numBonds, 5)
    self.assertEqual(res.smartsString, "[#6&R]1-&@[#6&R]-&@[#6&R]-&@[#6&R]-&@[#6&R]-&@1")

    params = rdFMCS.MCSParameters()
    params.AtomCompareParameters.CompleteRingsOnly = True
    # this will automatically be set to True
    params.BondCompareParameters.CompleteRingsOnly = False
    res = rdFMCS.FindMCS(mols, params)
    self.assertEqual(res.numAtoms, 5)
    self.assertEqual(res.numBonds, 5)
    self.assertEqual(res.smartsString, "[#6&R]1-&@[#6&R]-&@[#6&R]-&@[#6&R]-&@[#6&R]-&@1")

  def test21ShouldAcceptMCS(self):
    mols1 = [
      Chem.MolFromSmiles(smi) for smi in [
        "OC(=O)CCc1c(C)ccc2ccccc12",
        "c1(C)cc(cccc2)c2c(*)c1"
      ]
    ]
    mols2 = [
      Chem.MolFromSmiles(smi) for smi in [
        "OC(=O)C(C1CC1)Cc1c(C)ccc2ccccc12",
        "c1(C)cc(cccc2)c2c(C2CC2*)c1"
      ]
    ]
    params = rdFMCS.MCSParameters()
    params.AtomTyper = Test21AtomCompare()
    params.BondTyper = rdFMCS.BondCompare.CompareAny
    self.assertRaises(TypeError,
                      lambda params: setattr(params, "ShouldAcceptMCS", Test21AcceptanceCallbackCallbackIsInt()))
    self.assertRaises(TypeError,
                      lambda params: setattr(params, "ShouldAcceptMCS", Test21AcceptanceCallbackNoCallback()))
    self.assertRaises(TypeError,
                      lambda params: setattr(params, "FinalMatchChecker", Test21FinalMatchCheckCallbackCallbackIsInt()))
    self.assertRaises(TypeError,
                      lambda params: setattr(params, "FinalMatchChecker", Test21FinalMatchCheckCallbackNoCallback()))
    params.ShouldAcceptMCS = Test21AcceptanceCallback(self)
    params.ProgressCallback = Test21ProgressCallback(self)
    params.FinalMatchChecker = Test21FinalMatchCheckCallback(self)
    params.AtomCompareParameters.RingMatchesRingOnly = False
    params.AtomCompareParameters.CompleteRingsOnly = False
    params.BondCompareParameters.RingMatchesRingOnly = True
    params.BondCompareParameters.CompleteRingsOnly = False
    mcs = rdFMCS.FindMCS(mols1, params)
    self.assertEqual(mcs.numAtoms, 11)
    self.assertEqual(mcs.numBonds, 12)
    self.assertEqual(mcs.smartsString, "[#6]1:&@[#6]:&@[#6]2:&@[#6]:&@[#6]:&@[#6]:&@[#6]:&@[#6]:&@2:&@[#6](:&@[#6]:&@1)-&!@[#0,#6]")
    self.assertGreater(params.ProgressCallback.callCount, 0)
    self.assertGreater(params.FinalMatchChecker.callCount, 0)
    params.AtomCompareParameters.RingMatchesRingOnly = True
    mcs = rdFMCS.FindMCS(mols2, params)
    self.assertEqual(mcs.numAtoms, 4)
    self.assertEqual(mcs.numBonds, 4)
    self.assertEqual(mcs.smartsString, "[#6]1-&@[#6]-&@[#6]-&@1-&!@[#0,#6;!R]")
    self.assertGreater(params.ProgressCallback.callCount, 0)
    self.assertGreater(params.FinalMatchChecker.callCount, 0)

  def test22DegenerateMCS(self):
    para = "CC1:C:C:C(N):C:C:1"
    ortho = "CC1:C:C:C:C:C:1N"
    if 1:
      mols1 = [Chem.MolFromSmiles(smi) for smi in [
        "Nc1ccc(cc1)C-Cc1c(N)cccc1",
        "Nc1ccc(cc1)C=Cc1c(N)cccc1"]
      ]
      mols2 = [Chem.MolFromSmiles(smi) for smi in [
        "Nc1ccccc1C-Cc1ccc(N)cc1",
        "Nc1ccccc1C=Cc1ccc(N)cc1"]
      ]
      p = rdFMCS.MCSParameters()
      mcs1 = rdFMCS.FindMCS(mols1)
      self.assertEqual(mcs1.numAtoms, 8)
      self.assertEqual(mcs1.numBonds, 8)
      self.assertNotEqual(mcs1.smartsString, "")
      self.assertIsNotNone(mcs1.queryMol)
      mcs1Smiles = Chem.MolToSmiles(Chem.MolFromSmarts(mcs1.smartsString))
      self.assertEqual(mcs1Smiles, para)
      self.assertEqual(len(mcs1.degenerateSmartsQueryMolDict), 0)
      mcs2 = rdFMCS.FindMCS(mols2)
      self.assertEqual(mcs2.numAtoms, 8)
      self.assertEqual(mcs2.numBonds, 8)
      self.assertNotEqual(mcs2.smartsString, "")
      self.assertIsNotNone(mcs2.queryMol)
      mcs2Smiles = Chem.MolToSmiles(Chem.MolFromSmarts(mcs2.smartsString))
      self.assertEqual(mcs2Smiles, ortho)
      self.assertEqual(len(mcs2.degenerateSmartsQueryMolDict), 0)
      self.assertNotEqual(mcs1Smiles, mcs2Smiles)
      self.assertEqual(Chem.MolToSmiles(mols1[0]), Chem.MolToSmiles(mols2[0]))
      self.assertEqual(Chem.MolToSmiles(mols1[1]), Chem.MolToSmiles(mols2[1]))
      p.StoreAll = True
      mcs1 = rdFMCS.FindMCS(mols1, p)
      self.assertEqual(mcs1.numAtoms, 8)
      self.assertEqual(mcs1.numBonds, 8)
      self.assertEqual(mcs1.smartsString, "")
      self.assertIsNone(mcs1.queryMol)
      self.assertEqual(len(mcs1.degenerateSmartsQueryMolDict), 2)
      mcs2 = rdFMCS.FindMCS(mols2, p)
      self.assertEqual(mcs2.numAtoms, 8)
      self.assertEqual(mcs2.numBonds, 8)
      self.assertEqual(mcs2.smartsString, "")
      self.assertIsNone(mcs2.queryMol)
      self.assertEqual(len(mcs2.degenerateSmartsQueryMolDict), 2)
      degSmiles1 = sorted(Chem.MolToSmiles(Chem.MolFromSmarts(sma)) for sma in mcs1.degenerateSmartsQueryMolDict.keys())
      degSmiles2 = sorted(Chem.MolToSmiles(Chem.MolFromSmarts(sma)) for sma in mcs2.degenerateSmartsQueryMolDict.keys())
      self.assertEqual(len(degSmiles1), 2)
      self.assertEqual(degSmiles1, degSmiles2)
    if 1:
      mols = [Chem.MolFromSmiles(smi) for smi in [
        "Nc1ccc(cc1)C-C",
        "Nc1ccc(cc1)C=C"]
      ]
      p = rdFMCS.MCSParameters()
      mcs1 = rdFMCS.FindMCS(mols)
      self.assertEqual(mcs1.numAtoms, 8)
      self.assertEqual(mcs1.numBonds, 8)
      self.assertIsNotNone(mcs1.queryMol)
      self.assertNotEqual(mcs1.smartsString, "")
      self.assertEqual(Chem.MolToSmiles(Chem.MolFromSmarts(mcs1.smartsString)), para)
      self.assertEqual(len(mcs1.degenerateSmartsQueryMolDict), 0)
      p.StoreAll = True
      mcs2 = rdFMCS.FindMCS(mols, p)
      self.assertEqual(mcs2.numAtoms, 8)
      self.assertEqual(mcs2.numBonds, 8)
      self.assertIsNone(mcs2.queryMol)
      self.assertEqual(mcs2.smartsString, "")
      self.assertEqual(len(mcs2.degenerateSmartsQueryMolDict), 1)
      self.assertEqual(Chem.MolToSmiles(Chem.MolFromSmarts(tuple(mcs2.degenerateSmartsQueryMolDict.keys())[0])), para)


if __name__ == "__main__":
  unittest.main()
